using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SpaceInvadersPhone
{
    public class BayesNetwork
    {
        public bool AboveBarrier = false, AbovePlayer = false, WithinRadius = false;

        public float fireTrue = 0.2f, fireFalse = 0.8f;
        public float aboveBarTrue = 0.25f, aboveBarFalse = 0.25f;
        public float abovePlayTrue = 0.75f, abovePlayFalse = 0.25f;
        public float withinRadTrue = 0.4f, withinRadFalse = 0.25f;
        public float Probability;

        public bool firstPart = false, secondPart = false;

        /// <summary>
        /// Creates the bayesian network
        /// </summary>
        /// <param name="aboveBarrier">Boolean to represent if the enemy is above a barrier block</param>
        /// <param name="abovePlayer">Boolean to represent if the enemy is above the player</param>
        /// <param name="withinRadius">Boolean to represent if the enemy is within a radius of the player</param>
        /// <param name="roundNumber"></param>
        public void generateNetwork(bool aboveBarrier, bool abovePlayer, bool withinRadius, int roundNumber)
        {
            AboveBarrier = aboveBarrier;
            AbovePlayer = abovePlayer;
            WithinRadius = withinRadius;

            //This calculation will dynamically update the probability of the enemies firing to allow for more agressive enemies
            //as the round number increases
            fireTrue = fireTrue * (roundNumber / 10 + fireTrue);

            Console.WriteLine("probability of firing = " + fireTrue);

            Probability = generateProbability();
        }

        /// <summary>
        /// Generate the probability of an action given the current state of 
        /// the bayesian network
        /// </summary>
        /// <returns>The probability of an action occurring</returns>
        public float generateProbability()
        {
            float probabilityIfTrue = 0, probabilityIfFalse = 0, probability = 0;

            firstPart = true;

            probabilityIfTrue = fireTrue * (abovePlayer(AbovePlayer, firstPart)) * (aboveBarrier(AboveBarrier, firstPart) * (withinRaddius(WithinRadius, firstPart)));

            firstPart = false;

            probabilityIfFalse = fireFalse * (abovePlayer(AbovePlayer, firstPart)) * (aboveBarrier(AboveBarrier, firstPart) * (withinRaddius(WithinRadius, firstPart)));

            probability = probabilityIfTrue / (probabilityIfTrue + probabilityIfFalse);

            Probability = probability;
            return probability;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="isAbovePlayer"></param>
        /// <param name="firstPart"></param>
        /// <returns></returns>
        public float abovePlayer(bool isAbovePlayer, bool firstPart)
        {
            if (isAbovePlayer == true)
            {
                return abovePlayTrue;
            }

            else
            {
                if (firstPart == true)
                {
                    return (1 - abovePlayTrue);
                }

                else
                {
                    return (1 - abovePlayFalse);
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="isAboveBarrier"></param>
        /// <param name="firstPart"></param>
        /// <returns></returns>
        public float aboveBarrier(bool isAboveBarrier, bool firstPart)
        {
            if (isAboveBarrier == true)
            {
                return aboveBarTrue;
            }

            else
            {
                if (firstPart == true)
                {
                    return (1 - aboveBarTrue);
                }

                else
                {
                    return (1 - aboveBarFalse);
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="isWithin"></param>
        /// <param name="firstPart"></param>
        /// <returns></returns>
        public float withinRaddius(bool isWithin, bool firstPart)
        {
            if (isWithin == true)
            {
                return withinRadTrue;
            }

            else
            {
                if (firstPart == true)
                {
                    return (1 - withinRadTrue);
                }

                else
                {
                    return (1 - withinRadFalse);
                }
            }
        }
    }
}
