using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SpaceInvadersPhone
{
    public class Enemy
    {
        /// <summary>
        /// The current state of the enemy
        /// </summary>
        public enum EnemyAIState
        {
            Idle,
            HasLineofSight,
            FireLaser,
            BreakAwayLeft,
            BreakAwayRight,
            CanBreakAwayLeft,
            CanBreakAwayRight,
            CantBreakLeft,
            CantBreakRight,
            ReturnToScreen,
            Destroyed
        }

        /// <summary>
        /// The previous state of the enemy
        /// </summary>
        public enum previousState
        {
            Idle,
            HasLineofSight,
            FireLaser,
            BreakAwayLeft,
            BreakAwayRight,
            CanBreakAwayLeft,
            CanBreakAwayRight,
            CantBreakLeft,
            CantBreakRight,
            ReturnToScreen,
            Destroyed
        }

        // Animation representing the player
        public Texture2D EnemyTexture;

        // Position of the Player relative to the upper left side of the screen
        public Vector2 Position;

        //Rectangle that scales the size of the enemy texture to equal 10% of the screen width and height
        public Rectangle enemyRect;

        // State of the enemy ship
        public bool Active;

        // Get the width of the enemy ship
        public int Width
        {
            get { return EnemyTexture.Width; }
        }

        // Get the height of the player ship
        public int Height
        {
            get { return EnemyTexture.Height; }
        }

        public float enemySpeed;

        //The number of points obtained from killing the enemy
        public int Value;

        public TimeSpan decisionTime;
        public TimeSpan previousDecisionTime;

        public EnemyAIState enemyState;

        public previousState prevState;

        //This value will be used to numerically identify an enemy in the list 
        public Vector2 Number;

        //Number of enemies present on screen
        //public int EnemyNo;

        //Initialise the first position of the enemy ship
        //X coordinate of enemy
        //public float xCoord = 350;

        //Y coordinate of the enemy
        // public float yCoord = 100;


        /// <summary>
        /// Used to make an enemy break away from the main pack
        /// </summary>
        public void breakAway()
        {
            enemyRect.Y = enemyRect.Y + 1;
            enemyRect.X = enemyRect.X - 1;
        }

        /// <summary>
        /// Change the state of an enemy given its previous state
        /// </summary>
        /// <param name="oldState">The state of the enemy before transition</param>
        /// <param name="newState">The new state</param>
        public void changeState(int oldState, int newState)
        {
            switch (newState)
            {
                case 1:

                    enemyState = EnemyAIState.Idle;
                    break;

                case 2:

                    enemyState = EnemyAIState.HasLineofSight;
                    break;

                case 3:

                    enemyState = EnemyAIState.FireLaser;
                    break;

                case 4:

                    enemyState = EnemyAIState.CanBreakAwayLeft;
                    break;

                case 5:

                    enemyState = EnemyAIState.CanBreakAwayRight;
                    break;

                case 6:

                    enemyState = EnemyAIState.CantBreakLeft;
                    break;

                case 7:

                    enemyState = EnemyAIState.CantBreakRight;
                    break;

                case 8:

                    enemyState = EnemyAIState.BreakAwayLeft;
                    break;

                case 9:

                    enemyState = EnemyAIState.BreakAwayRight;
                    break;

                case 10:

                    enemyState = EnemyAIState.ReturnToScreen;
                    break;

                case 11:

                    enemyState = EnemyAIState.Destroyed;
                    break;
            }

            switch (oldState)
            {
                case 1:

                    prevState = previousState.Idle;
                    break;

                case 2:

                    prevState = previousState.HasLineofSight;
                    break;

                case 3:

                    prevState = previousState.FireLaser;
                    break;

                case 4:

                    prevState = previousState.CanBreakAwayLeft;
                    break;

                case 5:

                    prevState = previousState.BreakAwayRight;
                    break;

                case 6:

                    prevState = previousState.CantBreakLeft;
                    break;

                case 7:

                    prevState = previousState.CantBreakRight;
                    break;

                case 8:

                    prevState = previousState.BreakAwayLeft;
                    break;

                case 9:

                    prevState = previousState.BreakAwayRight;
                    break;

                case 10:

                    prevState = previousState.ReturnToScreen;
                    break;

                case 11:

                    prevState = previousState.Destroyed;
                    break;
            }
        }

        /// <summary>
        /// This is the base class constructor for the enemy
        /// </summary>
        /// <param name="texture">The enemy graphic</param>
        /// <param name="level">The level the player is currently on</param>
        /// <param name="position">The desired position of the enemy</param>
        /// <param name="viewport"></param>
        /// <param name="arrayPosition">The position of the enemy in 2D Jagged array</param>
        public void Initialize(Texture2D texture, int level, Vector2 position, Viewport viewport, Vector2 arrayPosition)
        {
            //TESTING BE GOING ON HERE
            Number = arrayPosition;

            //Set the AI state of the enemy
            enemyState = EnemyAIState.Idle;
            prevState = previousState.Idle;

            //Load enemy ship texture
            EnemyTexture = texture;

            //Set the position of the enemy 
            Position = position;

            //Set speed of enemy movement
            enemySpeed = 1f;

            //If the enemy speed hasnt reached its maximum yet then
            if (enemySpeed > 0.2f)
            {
                //As the number of rounds the user progresses, the faster the enemies movement becomes
                enemySpeed = enemySpeed - ((float)level / 10);
            }

            Active = true;

            Value = 100;

            decisionTime = TimeSpan.FromSeconds(1f);

            float enemyAspectRatio = (float)EnemyTexture.Width / EnemyTexture.Height;

            float enemyWidth = (float)viewport.Width / 40;

            float enemyHeight = enemyWidth / enemyAspectRatio;

            //Initialise the size of the player rectangle
            enemyRect = new Rectangle((int)Position.X, (int)Position.Y, (int)enemyWidth, (int)enemyHeight);
        }

        /// <summary>
        /// Move the enemy to the left
        /// </summary>
        public void UpdateLeft(Viewport viewport)
        {
            enemyRect.X -= viewport.Width / 30;
        }

        /// <summary>
        /// Move the enemy to the right
        /// </summary>
        public void UpdateRight(Viewport viewport)
        {
            enemyRect.X += viewport.Width / 30;
        }

        /// <summary>
        /// Move the enemy down 
        /// </summary>
        public void UpdateDown(Viewport viewport)
        {
            enemyRect.Y += viewport.Height / 30;
        }

        /// <summary>
        /// Draw the enemey to the game screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(EnemyTexture, enemyRect, Color.White);
        }
    }

    /// <summary>
    ///This is a specialised subclass of the enemy super class, it is the most common enemy that will be contained in the lowest two rows
    /// </summary>
    class BadGuy : Enemy
    {
        /// <summary>
        /// This is an overloaded constructor used to initialise the basic enemy object
        /// </summary>
        /// <param name="texture">The enemy graphic</param>
        /// <param name="level">The level the player is currently on</param>
        /// <param name="position">The desired position of the enemy</param>
        /// <param name="viewport"></param>        
        public void Initialize(Texture2D texture, int level, Vector2 position, Viewport viewport)
        {
            //Load enemy ship texture
            EnemyTexture = texture;

            //Set the position of the enemy 
            Position = position;

            //Set speed of enemy movement
            enemySpeed = 0.2f;

            //If the enemy speed hasnt reached its maximum yet then
            if (enemySpeed > 0.2f)
            {
                //As the number of rounds the user progresses, the faster the enemies movement becomes
                enemySpeed = enemySpeed - ((float)level / 10);
            }

            Active = true;

            Value = 10;

            decisionTime = TimeSpan.FromSeconds(1f);

            float enemyAspectRatio = (float)EnemyTexture.Width / EnemyTexture.Height;

            float enemyWidth = (float)viewport.Width / 35;

            float enemyHeight = enemyWidth / enemyAspectRatio;

            //Initialise the size of the player rectangle
            enemyRect = new Rectangle((int)Position.X, (int)Position.Y, (int)enemyWidth, (int)enemyHeight);
        }
    }

    /// <summary>
    /// This enemy is the next most common and will be found in the next two rows, above the common enemies
    /// </summary>
    class BadGuy1 : Enemy
    {       
        public void Initialize(Texture2D texture, int level, Vector2 position, Viewport viewport)
        {
            //Load enemy ship texture
            EnemyTexture = texture;

            //Set the position of the enemy 
            Position = position;

            //Set speed of enemy movement
            enemySpeed = 0.2f;

            //If the enemy speed hasnt reached its maximum yet then
            if (enemySpeed > 0.2f)
            {
                //As the number of rounds the user progresses, the faster the enemies movement becomes
                enemySpeed = enemySpeed - ((float)level / 10);
            }

            Active = true;

            Value = 20;

            decisionTime = TimeSpan.FromSeconds(1f);

            float enemyAspectRatio = (float)EnemyTexture.Width / EnemyTexture.Height;

            float enemyWidth = (float)viewport.Width / 35;

            float enemyHeight = enemyWidth / enemyAspectRatio;

            //Initialise the size of the player rectangle
            enemyRect = new Rectangle((int)Position.X, (int)Position.Y, (int)enemyWidth, (int)enemyHeight);
        }
    }

    /// <summary>
    /// This enemy will be containe din the top row fo the pack and will be wroth the most points 
    /// </summary>
    class BadGuy2 : Enemy
    {
        public void Initialize(Texture2D texture, int level, Vector2 position, Viewport viewport)
        {
            //Load enemy ship texture
            EnemyTexture = texture;

            //Set the position of the enemy 
            Position = position;

            //Set speed of enemy movement
            enemySpeed = 0.2f;

            //If the enemy speed hasnt reached its maximum yet then
            if (enemySpeed > 0.2f)
            {
                //As the number of rounds the user progresses, the faster the enemies movement becomes
                enemySpeed = enemySpeed - ((float)level / 10);
            }

            Active = true;

            Value = 40;

            decisionTime = TimeSpan.FromSeconds(1f);

            float enemyAspectRatio = (float)EnemyTexture.Width / EnemyTexture.Height;

            float enemyWidth = (float)viewport.Width / 35;

            float enemyHeight = enemyWidth / enemyAspectRatio;

            //Initialise the size of the player rectangle
            enemyRect = new Rectangle((int)Position.X, (int)Position.Y, (int)enemyWidth, (int)enemyHeight);
        }
    }

    /// <summary>
    /// This enemy will appear at random during the game and will either travel from the left/right of the screen
    /// </summary>
    class RareEnemy : Enemy
    {
        public int Direction;

        public Random random = new Random();

        public void Initialize(Texture2D texture, Viewport viewport)
        {
            //Load enemy ship texture
            EnemyTexture = texture;

            //Set speed of enemy movement
            enemySpeed = (float)random.Next(1, 5);

            Value = (int)enemySpeed * 100;

            //Randomly determine the direction of the enemy
            Direction = RandomDirection();
            
            decisionTime = TimeSpan.FromSeconds(1f);

            float enemyAspectRatio = (float)EnemyTexture.Width / EnemyTexture.Height;

            float enemyWidth = (float)viewport.Width / 40;

            float enemyHeight = enemyWidth / enemyAspectRatio;

            if (Direction == 1)
            {
                Position.X = 0;
                Position.Y = viewport.Height / 11;
            }

            if (Direction == 2)
            {
                Position.X = viewport.Width;
                Position.Y = viewport.Height / 11;
            }

            //Initialise the size of the player rectangle
            enemyRect = new Rectangle((int)Position.X, (int)Position.Y, (int)enemyWidth, (int)enemyHeight);
        }


        //Method to create a random direction with a 50% of either left or right, makes use of RandomBetween function, this was also taken from XNA Wiki
        /// <summary>
        /// Create a random direction with a 50% change of being either left or right
        /// </summary>
        /// <returns>1 to move from left to right or 2 to move from right to left</returns>
        public int RandomDirection()
        {
            int mysteryDirection;

            mysteryDirection = random.Next(1, 3);

            if (mysteryDirection == 2)
            {
                //Make the direction of the enemy move from right to left
                return 2;
            }

            else
            {
                //Make the direction of the enemy move from left to right
                return 1;
            }
        }

        /// <summary>
        /// Move the rare enemy left
        /// </summary>
        public void UpdateLeft()
        {
            //The enemy always moves to the left so decrement it's xposition           
            enemyRect.X -= (int)enemySpeed;
        }

        /// <summary>
        /// Move the rare enemy right 
        /// </summary>
        public void UpdateRight()
        {
            enemyRect.X += (int)enemySpeed;
        }

        /// <summary>
        /// Draw the rare enemy to the game screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(EnemyTexture, enemyRect, Color.White);
        }
    }
}
