using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SpaceInvadersPhone
{
    public class EnemyLaser
    {
        // Texture representing the laser
        public Texture2D Texture;

        // Position of the laser relative to the upper left side of the screen
        public Vector2 Position;

        // State of the laser
        public bool Active;

        //Represents the viewable boundary of the game window
        Viewport viewport;

        //Bounding box to put the laser texture in
        public Rectangle enemyLaserRect;

        //Get the width of the player ship
        public int Width
        {
            get { return Texture.Width; }
        }

        //Get the height of the player ship
        public int Height
        {
            get { return Texture.Height; }
        }

        //Speed of laser
        float laserSpeed;

        /// <summary>
        /// This is the class constructor
        /// </summary>
        /// <param name="viewport"></param>
        /// <param name="texture">The texture for the laser</param>
        /// <param name="position">The position of the laser relative to the position of the enemy that fired it</param>
        public void Initialize(Viewport viewport, Texture2D texture, Vector2 position)
        {
            Texture = texture;
            Position = position;
            this.viewport = viewport;

            Active = true;

            laserSpeed = 5f;

            float enemyLaserAspectRatio = (float)Width / Height;

            float enemyLaserWidth = (float)viewport.Width / 200;

            float enemyLaserHeight = enemyLaserWidth / enemyLaserAspectRatio;

            //Initialise the size of the player rectangle
            enemyLaserRect = new Rectangle((int)Position.X, (int)Position.Y, (int)enemyLaserWidth, (int)enemyLaserHeight);
        }

        /// <summary>
        /// Update the laser position so that it travels down the screen
        /// </summary>
        public void Update()
        {
            //The laser will head downwards towards the player rather than upwards
            enemyLaserRect.Y += (int)laserSpeed;

            if (enemyLaserRect.Y > viewport.Height)
            {
                Active = false;
            }
        }

        /// <summary>
        /// Draw the laser to the game screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(Texture, enemyLaserRect, Color.White);
        }
    }
}