using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using System.Diagnostics;

namespace SpaceInvadersPhone
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //Used to store the game states
        Boolean SplashScreen, Menu, GameOver, Play, Instructions, Exit, Controls, Paused;
       
        //The font type used in game
        SpriteFont font, font1, pausedFont;

        // Represents the player 
        Player player;
        Texture2D playerTexture, playerTexture1, playerTexture2, playerTextureShield, playerTexture1Shield, playerTexture2Shield, BorderTexture;

        //Represents the enemy
        Enemy enemy1;

        //Will need to make a list of enemies instead soon to then implement collision detection
        List<Enemy> enemies;

        //List of power ups should there be more than one currently active on screen
        List<PowerUp> powerUps;

        //The enemy textures
        Texture2D enemyTexture1, enemyTexture2, enemyTexture3, enemyTexture4;

        //The interval of time it takes for the enemy to move
        float enemyMovement;

        //The y position of the respawn point for the enemies
        //int lastY = 50;
        int lastY;

        //Variable to depict the start of the round and when the enemies are added to the game
        Boolean startOfRound = true;

        //Variable to depict the end of a round as the user has successfully defeated all present enemies
        Boolean endOfRound = false;

        //The level number the user is currently playing
        int levelNo = 1;

        //Represents the menu
        Menu menu;

        // Keyboard states used to determine key presses
        KeyboardState currentKeyboardState;
        KeyboardState previousKeyboardState;

        // Gamepad states used to determine button presses
        GamePadState currentGamePadState;
        GamePadState previousGamePadState;

        //Texture to represent the laser
        Texture2D laserTexture;

        //List variable that acts like an array but is dynamic will store any and all lasers present
        List<Laser> lasers;
        Laser laser1;

        //The interval of time between which the user has to wait before being allowed to fire again
        TimeSpan fireTime;
        TimeSpan previousFireTime;

        //The interval of time between enemy movements
        TimeSpan currentMovementTime;
        TimeSpan previousMovementTime;

        //Enemy movement flags
        Boolean moveLeft, moveRight;

        //Values to keep track of the enemy in the top left and right hand corner of the game
        int topLeftCount = 0, topRightCount = 4;

        //Texture to represent the enemy lasers
        Texture2D enemyLaserTexture;

        //List variable to store all enemy lasers present 
        List<EnemyLaser> enemyLasers;
        EnemyLaser enemyLaser1;
      
        //Represents the barrier class
        Barrier barrierBlock;
        List<Barrier> barrier;

        //Represents how many barriers have been drawn, used as the condition to satisfy recursive barrier initialise method
        int barrierCount = 0;

        //Stores the coordinates for the first barrier block
        Vector2 barrierPos;

        //Texture to represent the barrier
        Texture2D barrierTexture, barrierLeftCornerTexture, barrierRightCornerTexture;

        //A value used to store exactly 1/8th of the viewport
        int eighthOfScreen;

        //Used to store the time it takes for enemy to make decisions to shoot
        public TimeSpan decisionTime;
        public TimeSpan previousDecisionTime;

        //Used to store the time interval between updating enemies that have broken away
        public TimeSpan curveMovementTime;
        public TimeSpan curvePreviousMovementTime;
        
        int[] reductionArray, biggestKilled;


        BadGuy badGuy;
        BadGuy1 badGuy1;
        BadGuy2 badGuy2;
        RareEnemy rareEnemy;

        //Value to store the amount of time the xbox controller is to vibrate given a particular situation
        TimeSpan startLaserFire;
        TimeSpan endLaserFire;
        TimeSpan startEnemyDestroyed;
        TimeSpan endEnemyDestroyed;
        TimeSpan startPlayerDestroyed;
        TimeSpan endPlayerDestroyed;

        Boolean isMovingLeft = false, isMovingRight = false;

        //This boolean is used to determine whether a conflict has been found in the enemy list
        Boolean conflict = true;

        int idle = 1, hasLineofSight = 2, fireLaser = 3, canBreakAwayLeft = 4, canBreakAwayRight = 5, cantBreakLeft = 6, cantBreakRight = 7,  breakAwayLeft = 8, breakAwayRight = 9, returnToScreen = 10, destroyed = 11;

        Instructions instructions;

        int[] nextToFire;

        struct EnemyBlock
        {
            int enemyNumber;
            int nextToFire;
        }

        int?[][] enemyBlock = new int?[5][];

        int incremental = 0;

        //Variables used to store the position of a left/right most enemy on screen for updating purposes
        int topLeftEnemyNo = 0, topRightEnemyNo = 9;


        int leftColumn = 0, rightColumn = 9;

        //Boolean variable used to determine if all enemies are in state destroyed
        Boolean allDead = false, doubleCheck = false;

        //These textures are for the different powerups that are dropped by defeated enemies 
        Texture2D powerUpShield;
        Texture2D powerUpMovement;
        Texture2D powerUpDouble;

        PowerUp powerUp;        
       
        //These two simple booleans will represent whether or not the player has opted to play the classic version of the game
        //or the modern version

        bool classic = false, modern = false;

        //Create instance of the bayes network class
        BayesNetwork bayes;

        //Create instance of the paused screen class
        PausedScreen paused; 

        //The background for the paused screen
        Texture2D pausedBackground; 

        //Boolean to show if the highscores have been updated yet
        bool first = true;

        //Boolean to ensure that only one of the players lives is deducted at a time
        bool condition = true;

        //Boolean to represent when to show the sub menu when play game has been clicked
        bool subMenu = false;
       
        //A list of coordinates that make up a curve for an enemy to follow if it breaks away
        List<Vector2> path;

        //A cap on the number of enemies that can break away at once
        double breakAwayCap = 1;

        //A list containing two vectors which will limit where on the screen an enemy can break away
        List<Vector2> Thresholds;        

        List<Vector2> curveCoordinates;        

        int breakingEnemyCount = 0;

        List<Vector2> catchUpCoordinates;

        int numberOfEnemiesBreaking = 0;

        PlayScreen playScreen;

        Texture2D background;

        ControlScreen controlScreen;

        bool rareEnemyActive = false;

        //Random number generator
        Random random = new Random();

        //Timers to keep track of how long a power up has been active
        TimeSpan shieldActivationTime;
        TimeSpan dualLasersActivationTime;
        TimeSpan movementActivationTime;

        //boolean to ensure the highscores file is updated once
        bool updateHighScores = false;

        //
        int shieldPower = 1, movementPower = 2, doubleLaser = 3;

        Texture2D splashScreen;

        SplashScreen splash;

        TimeSpan splashScreenTimer;        

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }
        
        protected override void Initialize()
        {
            splashScreenTimer = TimeSpan.FromSeconds(1f);

            playScreen = new PlayScreen();

            controlScreen = new ControlScreen();

            //List to store lists of curved path coordinates
            curveCoordinates = new List<Vector2>();

            //The coordinates for the enemy to return to if it is not destroyed when breaking away
            catchUpCoordinates = new List<Vector2>();

            for (int j = 0; j < enemyBlock.Length; j++)
            {
                enemyBlock[j] = new int?[10];
            }

            //Create instance of the player class
            player = new Player();

            //Create instance of the enemy class
            enemy1 = new Enemy();
            enemies = new List<Enemy>();

            //Create instance of the menu class
            menu = new Menu();

            //Create instance of the the instructions class
            instructions = new Instructions();            
            

            //Create instance of the laser class
            lasers = new List<Laser>();
            laser1 = new Laser();

            //Set the laser to fire every half a second
            fireTime = TimeSpan.FromSeconds(.5f);

            //Create instance of the enemyLaser class 
            enemyLasers = new List<EnemyLaser>();

            //Create instance of the paused class
            paused = new PausedScreen();

            //Create instance of the splash screen class
            splash = new SplashScreen();

            //Create instance of the power up class
            powerUp = new PowerUp();            

            //Create a new power up list
            powerUps = new List<PowerUp>();

            //Create instance of the bayes network class
            bayes = new BayesNetwork();

            //Create instance of the barrier object
            barrierBlock = new Barrier();
            barrier = new List<Barrier>();

            eighthOfScreen = (GraphicsDevice.Viewport.Width / 4) - ((GraphicsDevice.Viewport.Width / 4) / 2);

            barrierPos.X = (int)(GraphicsDevice.Viewport.Width / 4) - ((GraphicsDevice.Viewport.Width / 4) / 2) - (barrierBlock.barrierRect.Width * 2);
            barrierPos.Y = (int)(GraphicsDevice.Viewport.Height / (float)1.15);
          
            //Initialise game states
            SplashScreen = true;
            Menu = false;
            GameOver = false;
            Play = false;
            Instructions = false;
            Exit = false;
            Controls = false;           
            Paused = false;

            //Initialise enemy movement phases
            moveLeft = true;
            moveRight = false;

            //Initialise Main Menu
            menu.Initialize(GraphicsDevice.Viewport);

            //Initialise the high scores
            //score.Initialize(GraphicsDevice.Viewport);

            //Initialise the instructions menu 
            instructions.Initialise(GraphicsDevice.Viewport);
            
            //Respawn position of the enemy in the top left hand corner
            lastY = (int)(GraphicsDevice.Viewport.Height / (float)7);

            decisionTime = TimeSpan.FromSeconds(0.5f);

            curveMovementTime = TimeSpan.FromSeconds(0.03f);

            endLaserFire = TimeSpan.FromSeconds(1f);

            reductionArray = new int[5];
            //reduction is a 5-element array used to monitor the number of enemies destroyed in each row

            

            //Initialise the array to contain all 10's as there are 10 enemies in each row 
            for (int i = 0; i < reductionArray.Length; i++)
            {
                reductionArray[i] = 10;                
            }

            biggestKilled = new int[5]; //this array remembers the highest number enemy killed in a particular row

            for (int i = 0; i < biggestKilled.Length; i++)
            {
                biggestKilled[i] = 0;
            }

            //Enable the FreeDrag gesture.
            TouchPanel.EnabledGestures = GestureType.HorizontalDrag | GestureType.Tap | GestureType.Hold;
           

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content such as graphical resources and fonts etc.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            splashScreen = Content.Load<Texture2D>("SplashScreen");
            splash.InitialiseScreen(splashScreen, GraphicsDevice.Viewport);

            background = Content.Load<Texture2D>("background");

            // Load the player resources            
            Vector2 playerPosition = new Vector2(GraphicsDevice.Viewport.TitleSafeArea.X, GraphicsDevice.Viewport.TitleSafeArea.Y + GraphicsDevice.Viewport.TitleSafeArea.Height / 2);
            Vector2 enemyPosition = new Vector2(GraphicsDevice.Viewport.TitleSafeArea.X, GraphicsDevice.Viewport.TitleSafeArea.Y + GraphicsDevice.Viewport.TitleSafeArea.Height / 2);


            playerTexture = Content.Load<Texture2D>("PlayerShip3");
            playerTexture1 = Content.Load<Texture2D>("bankingleft2");
            playerTexture2 = Content.Load<Texture2D>("bankingright2");


            playerTextureShield = Content.Load<Texture2D>("PlayerShield");
            playerTexture1Shield = Content.Load<Texture2D>("bankingLeftShield");
            playerTexture2Shield = Content.Load<Texture2D>("bankingRightShield");

            enemyTexture1 = Content.Load<Texture2D>("Enemy1");
            enemyTexture2 = Content.Load<Texture2D>("Enemy2");
            enemyTexture3 = Content.Load<Texture2D>("Enemy3");
            enemyTexture4 = Content.Load<Texture2D>("MysteryEnemy");

            laserTexture = Content.Load<Texture2D>("Laser2");

            enemyLaserTexture = Content.Load<Texture2D>("EnemyLaser");

            font = Content.Load<SpriteFont>("Font");
            font1 = Content.Load<SpriteFont>("Font1");
            pausedFont = Content.Load<SpriteFont>("Font1");

            barrierTexture = Content.Load<Texture2D>("BarrierBrick(Basic)");
            barrierLeftCornerTexture = Content.Load<Texture2D>("BarrierBrickLeftCorner(Basic)");
            barrierRightCornerTexture = Content.Load<Texture2D>("BarrierBrickRightCorner(Basic)");


            powerUpShield = Content.Load<Texture2D>("powerup_shield");
            powerUpMovement = Content.Load<Texture2D>("powerup_movement");
            powerUpDouble = Content.Load<Texture2D>("powerup_double");
            BorderTexture = Content.Load<Texture2D>("Border");    

            //Initialise the paused screen           
            paused.Initialize(GraphicsDevice.Viewport, laserTexture);

            //Initialise the game screen
            playScreen.Initialize(BorderTexture, GraphicsDevice.Viewport);

           //Initialise the instructions screen
            instructions.initialiseRectangles(enemyTexture1, enemyTexture2, enemyTexture3, enemyTexture4, powerUpShield, powerUpMovement, powerUpDouble, laserTexture);

            //Initilaise the control screen
            controlScreen.Initialize(GraphicsDevice.Viewport, laserTexture);        
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here     
        }

        /// <summary>
        /// Initialise the jagged array which implements a double link list for storing the position of an enemy along with a
        /// pointer to whom is to fire next upon destruction
        /// </summary>
        public void InitialiseEnemyArray()
        {
            int testVal = 0, testVal2 = 10, testVal3 = 20, testVal4 = 30;
            int? tests = null;

            for (int i = 0; i < enemyBlock.Length; i++)
            {
                for (int j = 0; j < enemyBlock[i].Length; j++)
                {
                    if (i == 0)
                    {
                        //Top row of enemies will have no pointers
                        enemyBlock[i][j] = tests;
                    }

                    if (i == 1)
                    {
                        enemyBlock[i][j] = testVal;
                        testVal++;
                    }

                    if (i == 2)
                    {
                        enemyBlock[i][j] = testVal2;
                        testVal2++;
                    }

                    if (i == 3)
                    {
                        enemyBlock[i][j] = testVal3;
                        testVal3++;
                    }

                    if (i == 4)
                    {
                        enemyBlock[i][j] = testVal4;
                        testVal4++;
                    }                  
                }           
            }
        }

        /// <summary>
        /// Main game loop, constantly called throughout running of the game and
        /// is used to update everything in the running game world
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Save the previous state of the keyboard and game pad so we can determinesingle key/button presses
            previousGamePadState = currentGamePadState;
            previousKeyboardState = currentKeyboardState;

            // Read the current state of the keyboard and gamepad and store it
            currentKeyboardState = Keyboard.GetState();

            currentGamePadState = GamePad.GetState(PlayerIndex.One);

            //If the user is currently playing the game
            if (Play == true)
            {
                while (TouchPanel.IsGestureAvailable)
                {
                    GestureSample gesture = TouchPanel.ReadGesture();

                    if (gesture.GestureType == GestureType.Hold)
                    {
                        Play = false;
                        Paused = true;
                    }
                }

                if (startOfRound == true && levelNo == 1)
                {                                               
                    AddPlayer();                    
                    AddEnemy(lastY);
                    AddBarriers(barrierPos);
                    AddRareEnemy();
                    barrierCount = 0;
                    startOfRound = false;
                    InitializeEnemyList();
                    InitialiseEnemyArray();

                    if (modern == true)
                    {
                        Thresholds = GenerateBreakAwayThresholds(new Vector2(enemies[0].enemyRect.X, enemies[0].enemyRect.Y), new Vector2(enemies[9].enemyRect.X, enemies[9].enemyRect.Y));
                    }
                }

                else if (GameOver == true)
                {
                    Play = false;
                    GameOver = true;                    
                }

                if (startOfRound == true && levelNo != 1)
                {
                    AddEnemy(lastY);
                    startOfRound = false;
                    InitializeEnemyList();
                    InitialiseEnemyArray();
                    breakAwayCap += 0.25;
                }

                //Update the player
                UpdatePlayer(gameTime);

                //Update the lasers
                UpdateLaser(gameTime);

                //Update the enemy
                UpdateEnemy(gameTime);

                //Update the barriers
                UpdateBarrier(gameTime);

                //Update any collisions that have occured
                UpdateCollision(gameTime);

                //Update the enemy lasers                
                UpdateEnemyLaser(gameTime);

                //Update the rare enemy
                UpdateRareEnemy(gameTime);

                //Update the power ups
                UpdatePowerUp(gameTime);

                //Update the power up timers
                UpdateTimers(gameTime);
            }

            if (SplashScreen == true)
            {        
                if (gameTime.TotalGameTime > splashScreenTimer)
                {
                    SplashScreen = false;
                    Menu = true;                    
                }               
            }
            
            //If the user is currently browsing the main menu
            if (Menu == true)
            {
                if (subMenu != true)
                {
                    while (TouchPanel.IsGestureAvailable)
                    {                        
                        GestureSample gesture = TouchPanel.ReadGesture();

                        if (gesture.GestureType == GestureType.Tap)
                        {
                            Vector2 tapPos = gesture.Position;

                            Rectangle touchRect = new Rectangle((int)tapPos.X, (int)tapPos.Y, GraphicsDevice.Viewport.Width / 50, GraphicsDevice.Viewport.Height / 50);                            

                            if (touchRect.Intersects(menu.rect))
                            {
                                menu.MenuPosition = 0;
                                MenuOperation(menu.MenuPosition);
                            }

                            if (touchRect.Intersects(menu.rect1))
                            {
                                menu.MenuPosition = 1;
                                MenuOperation(menu.MenuPosition);
                            }

                            if (touchRect.Intersects(menu.rect2))
                            {
                                menu.MenuPosition = 2;
                                MenuOperation(menu.MenuPosition);
                            }

                            if (touchRect.Intersects(menu.rect3))
                            {
                                menu.MenuPosition = 3;
                                MenuOperation(menu.MenuPosition);
                            }
                        }
                    }
                }

                else
                {
                    while (TouchPanel.IsGestureAvailable)
                    {
                        GestureSample gesture = TouchPanel.ReadGesture();

                        if (gesture.GestureType == GestureType.Tap)
                        {
                            Vector2 tapPos = gesture.Position;

                            Rectangle touchRect = new Rectangle((int)tapPos.X, (int)tapPos.Y, GraphicsDevice.Viewport.Width / 50, GraphicsDevice.Viewport.Height / 50);
                            if (touchRect.Intersects(menu.subRect))
                            {
                                classic = true;
                                Play = true;
                                Menu = false;
                                Instructions = false;
                                Controls = false;
                            }

                            if (touchRect.Intersects(menu.subRect1))
                            {
                                modern = true;
                                Play = true;
                                Menu = false;
                                Instructions = false;
                                Controls = false;
                            }

                            if (touchRect.Intersects(menu.subRect2))
                            {
                                subMenu = false;
                                menu.Initialize(GraphicsDevice.Viewport);
                            }
                        }
                    }
                }                
            }

            if (Instructions == true)
            {

                //If at any point the user wishes to quit the current game and return to the menu by pressing the Escape button
                while (TouchPanel.IsGestureAvailable)
                {
                    GestureSample gesture = TouchPanel.ReadGesture();

                    if (gesture.GestureType == GestureType.Tap)
                    {
                        Vector2 tapPos = gesture.Position;

                        Rectangle touchRect = new Rectangle((int)tapPos.X, (int)tapPos.Y, GraphicsDevice.Viewport.Width / 50, GraphicsDevice.Viewport.Height / 50);

                        if (touchRect.Intersects(instructions.returnToMenuRect))
                        {
                            ReturnToMenu();
                        }
                    }
                }
            }
            

            if (Paused == true)
            {
                //If the user has paused the game then show the paused game screen above the game screen
                //along with prompts to ask user what they want to do


                //If at any point the user wishes to quit the current game and return to the menu by pressing the Escape button
                if (currentKeyboardState.IsKeyDown(Keys.Back) || currentGamePadState.Buttons.B == ButtonState.Pressed)
                {
                    //Pause the game and ask whether the user wishes to continue, go back to the main screen,
                    //or exit the game all together

                    NewGame();
                }

                //If at any point the user wishes to quit the current game and return to the menu by pressing the Escape button
                if (currentKeyboardState.IsKeyDown(Keys.Enter) && previousKeyboardState.IsKeyUp(Keys.Enter) || currentGamePadState.Buttons.A == ButtonState.Pressed && previousGamePadState.Buttons.A == ButtonState.Released)
                {
                    //Pause the game and ask whether the user wishes to continue, go back to the main screen,
                    //or exit the game all together

                    Paused = false;
                    Play = true;
                }
            }

            if (Controls == true)
            {
                //If at any point the user wishes to quit the current game and return to the menu by pressing the Escape button
                while (TouchPanel.IsGestureAvailable)
                {
                    GestureSample gesture = TouchPanel.ReadGesture();

                    if (gesture.GestureType == GestureType.Tap)
                    {
                        Vector2 tapPos = gesture.Position;

                        Rectangle touchRect = new Rectangle((int)tapPos.X, (int)tapPos.Y, GraphicsDevice.Viewport.Width / 50, GraphicsDevice.Viewport.Height / 50);

                        if (touchRect.Intersects(controlScreen.rect))
                        {
                            ReturnToMenu();
                        }
                    }
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// Handles the input from the user in the menu screen
        /// </summary>
        /// <param name="position">Position of the highlighted text upon being selected</param>
        public void MenuOperation(int position)
        {
            if (subMenu != true)
            {
                if (position == 0)
                {
                    subMenu = true;
                    menu.ExpandMenuOption(GraphicsDevice.Viewport);
                }

                if (position == 1)
                {
                    Play = false;
                    Menu = false;
                    Instructions = true;
                    Controls = false;
                }

                if (position == 2)
                {
                    Play = false;
                    Menu = false;
                    Instructions = false;
                    Controls = true;
                    Exit = false;
                    Quit(Exit);
                }

                if (position == 3)
                {
                    Play = false;
                    Menu = false;
                    Instructions = false;
                    Controls = false;
                    Exit = true;
                    Quit(Exit);
                }
            }            
        }

        /// <summary>
        /// Add a player object to the game 
        /// </summary>
        public void AddPlayer()
        {
            player.Initialize(playerTexture, playerTexture1, playerTexture2, playerTextureShield, playerTexture1Shield, playerTexture2Shield, GraphicsDevice.Viewport);
        }

        /// <summary>
        /// Update the players position 
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdatePlayer(GameTime gameTime)
        {
            // Windows Phone Controls
            while (TouchPanel.IsGestureAvailable)
            {
                GestureSample gesture = TouchPanel.ReadGesture();

                if (gesture.GestureType == GestureType.HorizontalDrag)
                {
                    player.Position += gesture.Delta;          
                }

                if (gesture.GestureType == GestureType.Tap)
                {                   
                    AddLaser(player.Position);
                }              

                //This line of code will move the scaled rectangle of the player, this will replace the above line when its ready                
                player.playerRect.X = (int)MathHelper.Clamp(player.playerRect.X, 0, GraphicsDevice.Viewport.Width - player.playerRect.Width);
                player.playerRect.Y = (int)MathHelper.Clamp(player.playerRect.Y, 0, GraphicsDevice.Viewport.Height - player.playerRect.Height);

                //Reset the player ship animation when hits the boundary of the screen, this avoids sticking to the move left/right texture
                if (player.playerRect.X <= 0 || player.playerRect.X >= GraphicsDevice.Viewport.Width - player.playerRect.Width)
                {
                    isMovingLeft = false;
                    isMovingRight = false;
                }

                //If the player has been defeated then
                if (player.Active == false && condition == true)
                {
                    condition = false;                  
                    GameOver = player.LoseLife();
                    PlayerRespwan();
                }
            }
        }

        /// <summary>
        /// Initialise the list of firing enemies to those on the bottom row as they have the line of sight 
        /// and the 6 enemies on either side of the pack that are eligible to break away
        /// </summary>
        public void InitializeEnemyList()
        {
            //Initialse the the top corner enemies
            topLeftEnemyNo = 0;
            topRightEnemyNo = 9;
                 
            for (int i = enemies.Count; i > enemies.Count - 10; i--)
            {
                enemies[i - 1].changeState(idle, hasLineofSight);                 
            }

            if (modern == true)
            {
                //Initialise the enemies that are eligible to break away from the main pack
                InitialiseEligibleBreakAwayEnemies();
            }
        }

        /// <summary>
        /// Calculate the smallest/biggest X value coordinate on screen where 
        /// bezier curve will fit within bounds of screen
        /// </summary>
        /// <param name="leftMostEnemyPos">Coordinate of the left most enemy</param>
        /// <param name="rightMostEnemyPos">Coordinate of the right most enemy</param>
        /// <returns>A list containing two vectors, the smallest coordinate where a bezier curve will fit and vice versa</returns>
        public List<Vector2> GenerateBreakAwayThresholds(Vector2 leftMostEnemyPos, Vector2 rightMostEnemyPos)
        {
            List<Vector2> constraintCoordinates = new List<Vector2>();
            List<Vector2> curvePoints = new List<Vector2>();

            Vector2 currentSmallest = leftMostEnemyPos, currentBiggest = rightMostEnemyPos;

            while (currentBiggest.X < GraphicsDevice.Viewport.Width - 10)
            {                
                curvePoints = GenerateCurveRight(rightMostEnemyPos, GraphicsDevice.Viewport);
                
                //Find the largest x coordinate in the lst of coordinates
                for (int i = 0; i < curvePoints.Count; i++)
                {
                    if (currentBiggest.X < curvePoints[i].X)
                    {
                        currentBiggest = curvePoints[i];                        
                    }
                }

                if (currentBiggest.X < GraphicsDevice.Viewport.Width - 10)
                {
                    rightMostEnemyPos.X += 10;
                }
            }

            constraintCoordinates.Add(rightMostEnemyPos);
          
            while (currentSmallest.X > 10)
            {
                GenerateCurveLeft(leftMostEnemyPos, GraphicsDevice.Viewport);

                //Find the smallest x coordinate in the list of coordinates
                for (int i = 0; i < path.Count; i++)
                {
                    if (currentSmallest.X > path[i].X)
                    {
                        currentSmallest = path[i];
                    }
                }

                if (currentSmallest.X > 10)
                {
                    leftMostEnemyPos.X -= 10;
                }
            }
           
            constraintCoordinates.Add(leftMostEnemyPos);
            return constraintCoordinates;
        }

        /// <summary>
        /// Initialise the enemies that are eligible to break away on the left hand side of the pack
        /// </summary>
        public void InitialiseEligibleBreakAwayEnemies()
        {
            enemies[10].changeState(idle, canBreakAwayLeft);
            enemies[19].changeState(idle, canBreakAwayRight);
        }

        /// <summary>
        /// Manage which enemies are eligible to break away given their position 
        /// </summary>
        public void BreakAwayManager()
        {
            for (int i = 0; i < enemies.Count; i++)
            {
                if (enemies[i].enemyState == Enemy.EnemyAIState.CanBreakAwayLeft || enemies[i].enemyState == Enemy.EnemyAIState.CantBreakLeft)
                {                    
                    //If the enemy position is greater than or equal to the minimum threshold then
                    if (enemies[i].enemyRect.X >= Thresholds[1].X)
                    {                        
                        enemies[i].changeState(cantBreakLeft, canBreakAwayLeft);                        
                    }

                    else
                    {
                        if (enemies[i].enemyState == Enemy.EnemyAIState.CanBreakAwayLeft)
                        {
                            enemies[i].changeState(canBreakAwayLeft, cantBreakLeft);                           
                        }
                    }
                }

                else if (enemies[i].enemyState == Enemy.EnemyAIState.CanBreakAwayRight || enemies[i].enemyState == Enemy.EnemyAIState.CantBreakRight)
                {

                    //If the enemy position is greater than or equal to the minimum threshold then
                    if (enemies[i].enemyRect.X <= Thresholds[0].X)
                    {
                        enemies[i].changeState(cantBreakRight, canBreakAwayRight);
                    }

                    else
                    {
                        if (enemies[i].enemyState == Enemy.EnemyAIState.CanBreakAwayRight)
                        {
                            enemies[i].changeState(canBreakAwayRight, cantBreakRight);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        public int UpdateBreakAwayEnemies(int count)
        {
            for (int j = 0; j < enemies.Count; j++)
            {
                //Generate curved paths for break away enemies
                if (enemies[j].enemyState == Enemy.EnemyAIState.BreakAwayLeft)
                {

                    if (j == topLeftEnemyNo)
                    {
                        FindLeftMostEnemy(j);
                    }

                    if (enemies[j].enemyRect.X + enemies[j].enemyRect.Width >= GraphicsDevice.Viewport.Width)
                    {
                        enemies[j].changeState(breakAwayLeft, returnToScreen);
                        enemies[j].enemyRect.X = GraphicsDevice.Viewport.Width / 2;
                        enemies[j].enemyRect.Y = 0;
                    }

                    else
                    {
                        if (count <= curveCoordinates.Count - 1)
                        {
                            enemies[j].enemyRect.X = (int)curveCoordinates[count].X;
                            enemies[j].enemyRect.Y = (int)curveCoordinates[count].Y;
                            break;
                        }
                    }
                }

                if (enemies[j].enemyState == Enemy.EnemyAIState.BreakAwayRight)
                {
                    if (j == topRightCount)
                    {
                        FindRightMostEnemy(j);
                    }

                    if (enemies[j].enemyRect.X <= 0)
                    {
                        enemies[j].changeState(breakAwayRight, returnToScreen);
                        enemies[j].enemyRect.X = GraphicsDevice.Viewport.Width / 2;
                        enemies[j].enemyRect.Y = 0;
                    }

                    else
                    {
                        if (count <= curveCoordinates.Count - 1)
                        {
                            enemies[j].enemyRect.X = (int)curveCoordinates[count].X;
                            enemies[j].enemyRect.Y = (int)curveCoordinates[count].Y;
                            break;
                        }
                    }
                }
            }

            count++;

            return count;
        }

        /// <summary>
        /// 
        /// </summary>
        public void UpdateFiringEnemies()
        {
            for (int i = 0; i < enemies.Count; i++)
            {
                if (enemies[i].enemyState == Enemy.EnemyAIState.FireLaser)
                {
                    //Add a laser at the enemies position
                    AddEnemyLaser(new Vector2(enemies[i].enemyRect.X + (enemies[i].enemyRect.Width / 2), enemies[i].enemyRect.Y + enemies[i].enemyRect.Height));
                    //Reset that enemies state back to has line of sight
                    enemies[i].changeState(fireLaser, hasLineofSight);
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public Vector2 UpdateReturnToScreenEnemies()
        {
            //The destination coordinate
            Vector2 catchUp = new Vector2();

            for (int g = 0; g < enemies.Count; g++)
            {
                if (enemies[g].enemyState == Enemy.EnemyAIState.ReturnToScreen)
                {
                    //Create a path for the enemy to follow to return to the screen
                    //Should be an easy ish calculation to do to the x,y coordinates to find its place

                    //First reset the enemies position to the top corner of the screen
                    //with respect to the direction it broke away
                    if (enemies[g].prevState == Enemy.previousState.BreakAwayLeft)
                    {
                        catchUp = new Vector2(enemies[g + 1].enemyRect.X - (enemies[g + 1].enemyRect.Width * 2), enemies[g + 1].enemyRect.Y);
                    }

                    if (enemies[g].prevState == Enemy.previousState.BreakAwayRight)
                    {
                        catchUp = new Vector2(enemies[g - 1].enemyRect.X + (enemies[g - 1].enemyRect.Width * 2), enemies[g - 1].enemyRect.Y);
                    }
                }
            }

            return catchUp;
        }

        /// <summary>
        /// Retreieve an enemies current state
        /// </summary>
        /// <param name="enemyNo">Enemy number</param>
        /// <returns>Current state in the form of an integer</returns>
        public int getCurrentState(int enemyNo)
        {
            int stateNumber = 0;

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.Idle)
            {
                stateNumber = 1;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.HasLineofSight)
            {
                stateNumber = 2;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.FireLaser)
            {
                stateNumber = 3;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.CanBreakAwayLeft)
            {
                stateNumber = 4;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.CanBreakAwayRight)
            {
                stateNumber = 5;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.CantBreakLeft)
            {
                stateNumber = 6;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.CantBreakRight)
            {
                stateNumber = 7;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.BreakAwayLeft)
            {
                stateNumber = 8;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.BreakAwayRight)
            {
                stateNumber = 9;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.ReturnToScreen)
            {
                stateNumber = 10;
            }

            if (enemies[enemyNo].enemyState == Enemy.EnemyAIState.Destroyed)
            {
                stateNumber = 11;
            }

            return stateNumber;
        }

        public void UpdateBreakAwayPointers(int enemyNo)
        {
            int? nextEnemy = 0;

            if (enemies[enemyNo].prevState == Enemy.previousState.CanBreakAwayLeft || enemies[enemyNo].prevState == Enemy.previousState.BreakAwayLeft)
            {
                nextEnemy = GetNextLeft(enemyNo);

                if (nextEnemy != null)
                {
                    int currentState = getCurrentState((int)nextEnemy);
                    enemies[(int)nextEnemy].changeState(currentState, cantBreakLeft);
                }
            }

            else if (enemies[enemyNo].prevState == Enemy.previousState.CanBreakAwayRight || enemies[enemyNo].prevState == Enemy.previousState.BreakAwayRight)
            {
                nextEnemy = GetNextRight(enemyNo);
                if (nextEnemy != null)
                {
                    int currentState = getCurrentState((int)nextEnemy);
                    enemies[(int)nextEnemy].changeState(currentState, cantBreakRight);
                }
            }

            numberOfEnemiesBreaking--;
            catchUpCoordinates.RemoveAt(0);
            breakingEnemyCount = 0;
        }


        public int? GetNextRight(int enemyNo)
        {
            int? nextEnemyNo = enemyNo++;

            bool findNext = false;

            while (findNext == false)
            {
                if (enemies[(int)nextEnemyNo].enemyState == Enemy.EnemyAIState.Idle)
                {
                    break;
                }

                else
                {
                    if (nextEnemyNo > 10)
                    {
                        nextEnemyNo--;
                    }

                    else
                    {
                        nextEnemyNo = null;
                        break;
                    }
                }
            }

            return nextEnemyNo;
        }

        public int? GetNextLeft(int enemyNo)
        {
            int? nextEnemyNo = enemyNo++;

            bool findNext = false;

            while (findNext == false)
            {
                if (enemies[(int)nextEnemyNo].enemyState == Enemy.EnemyAIState.Idle)
                {
                    break;
                }

                else
                {
                    if (nextEnemyNo < 19)
                    {
                        nextEnemyNo++;
                    }

                    else
                    {
                        nextEnemyNo = null;
                        break;
                    }
                }
            }

            return nextEnemyNo;
        }

        /// <summary>
        /// Update the enemies on the screen
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateEnemy(GameTime gameTime)
        {
            //Allow the enemy to move every certain amount of time which is influenced on the round number the user has achieved
            //Round 1 value of time is 1 second
            currentMovementTime = TimeSpan.FromSeconds(enemyMovement);

            Vector2 catchUpCoordinate = new Vector2();

            if (endOfRound == false)
            {
                //This block of code tells the enemies with the state fire laser what to do
                if (gameTime.TotalGameTime - previousDecisionTime > decisionTime)
                {
                    int noOfEnemiesFiringLimit = levelNo;
                    int numberOfEnemiesFiringClassic = 0, numberOfEnemiesFiringModern = 0;

                    int probability = 100 - (levelNo * 2);

                    int mysteryRand = random.Next(0, 100);

                    if (rareEnemyActive == false)
                    {
                        if (mysteryRand >= 98)
                        {
                            AddRareEnemy();
                        }
                    }

                    //This section of code will give all of the enemies with the line of sight of player a chance to fire at them 
                    for (int g = 0; g < enemies.Count; g++)
                    {
                        if (enemies[g].enemyState == Enemy.EnemyAIState.HasLineofSight)
                        {

                            //If the classic version of the game has been selected, chose firing enemies at random with no AI
                            if (classic == true)
                            {
                                int rand = random.Next(0, 100);

                                if (rand >= probability)
                                {
                                    if (numberOfEnemiesFiringClassic != noOfEnemiesFiringLimit)
                                    {
                                        enemies[g].changeState(hasLineofSight, fireLaser);
                                        numberOfEnemiesFiringClassic++;
                                    }
                                }
                            }

                            //If the modern version of the game has been selected, chose firing enemies using bayesian network
                            if (modern == true)
                            {
                                UpdateBayesNetwork(g);
                                bayes.generateProbability();

                                double rand = random.NextDouble();


                                if (rand <= bayes.Probability)
                                {
                                    if (numberOfEnemiesFiringModern != noOfEnemiesFiringLimit)
                                    {
                                        enemies[g].changeState(hasLineofSight, fireLaser);
                                        numberOfEnemiesFiringModern++;
                                    }
                                }
                            }
                        }

                        if (enemies[g].enemyState == Enemy.EnemyAIState.CanBreakAwayLeft || enemies[g].enemyState == Enemy.EnemyAIState.CanBreakAwayRight)
                        {
                            int rand = random.Next(0, 100);

                            if (rand >= 100 - levelNo * 5)
                            {                               
                                if (numberOfEnemiesBreaking != breakAwayCap)
                                {
                                    if (enemies[g].enemyState == Enemy.EnemyAIState.CanBreakAwayLeft)
                                    {
                                        enemies[g].changeState(canBreakAwayLeft, breakAwayLeft);

                                        //Add original coordinates to a list so the enemy can find its place if it survives
                                        catchUpCoordinates.Add(new Vector2(enemies[g].enemyRect.X, enemies[g].enemyRect.Y));


                                        curveCoordinates = GenerateCurveLeft(new Vector2(enemies[g].enemyRect.X, enemies[g].enemyRect.Y), GraphicsDevice.Viewport);


                                        numberOfEnemiesBreaking++;
                                    }

                                    if (enemies[g].enemyState == Enemy.EnemyAIState.CanBreakAwayRight)
                                    {
                                        enemies[g].changeState(canBreakAwayRight, breakAwayRight);

                                        //Add original coordinates to a list so the enemy can find its place if it survives
                                        catchUpCoordinates.Add(new Vector2(enemies[g].enemyRect.X, enemies[g].enemyRect.Y));

                                        //Generate a bezier curve for the enemy to follow
                                        curveCoordinates = GenerateCurveRight(new Vector2(enemies[g].enemyRect.X, enemies[g].enemyRect.Y), GraphicsDevice.Viewport);

                                        numberOfEnemiesBreaking++;
                                    }
                                }
                            }
                        }
                    }

                    previousDecisionTime = gameTime.TotalGameTime;
                }

                if (gameTime.TotalGameTime - curvePreviousMovementTime > curveMovementTime)
                {
                    breakingEnemyCount = UpdateBreakAwayEnemies(breakingEnemyCount);

                    catchUpCoordinate = UpdateReturnToScreenEnemies();

                    for (int i = 0; i < enemies.Count; i++)
                    {
                        if (enemies[i].enemyState == Enemy.EnemyAIState.ReturnToScreen)
                        {
                            if (enemies[i].prevState == Enemy.previousState.BreakAwayLeft)
                            {
                                if (enemies[i].enemyRect.X > catchUpCoordinate.X)
                                {
                                    if (enemies[i].enemyRect.X != catchUpCoordinate.X)
                                    {
                                        enemies[i].enemyRect.X -= 5;
                                    }
                                }

                                else
                                {
                                    if (enemies[i].enemyRect.X < catchUpCoordinate.X)
                                    {
                                        enemies[i].enemyRect.X += 1;
                                    }
                                }

                                if (enemies[i].enemyRect.Y < catchUpCoordinate.Y)
                                {
                                    if (enemies[i].enemyRect.Y != catchUpCoordinate.Y)
                                    {
                                        enemies[i].enemyRect.Y += 1;
                                    }
                                }

                                if (enemies[i].enemyRect.X == catchUpCoordinate.X && enemies[i].enemyRect.Y == catchUpCoordinate.Y)
                                {
                                    enemies[i].changeState(returnToScreen, cantBreakLeft);
                                    numberOfEnemiesBreaking--;
                                    catchUpCoordinates.RemoveAt(0);
                                    breakingEnemyCount = 0;
                                }
                            }

                            else if (enemies[i].prevState == Enemy.previousState.BreakAwayRight)
                            {
                                if (enemies[i].enemyRect.X > catchUpCoordinate.X)
                                {
                                    if (enemies[i].enemyRect.X != catchUpCoordinate.X)
                                    {
                                        enemies[i].enemyRect.X -= 1;
                                    }
                                }

                                else
                                {
                                    if (enemies[i].enemyRect.X != catchUpCoordinate.X)
                                    {
                                        enemies[i].enemyRect.X += 5;
                                    }
                                }

                                if (enemies[i].enemyRect.Y < catchUpCoordinate.Y)
                                {
                                    if (enemies[i].enemyRect.Y != catchUpCoordinate.Y)
                                    {
                                        enemies[i].enemyRect.Y += 2;
                                    }
                                }

                                if (enemies[i].enemyRect.X == catchUpCoordinate.X && enemies[i].enemyRect.Y == catchUpCoordinate.Y)
                                {
                                    enemies[i].changeState(returnToScreen, cantBreakRight);
                                    numberOfEnemiesBreaking--;
                                    catchUpCoordinates.RemoveAt(0);
                                    breakingEnemyCount = 0;
                                }
                            }
                        }
                    }

                    curvePreviousMovementTime = gameTime.TotalGameTime;
                }

                UpdateFiringEnemies();

                //Keeps tabs on one of the most left most enemies so it knows when it has hit the left most wall of the game screen in order to know when to move down
                //and change direction                   
                if (enemies[topLeftEnemyNo].enemyRect.X <= 0 && moveLeft == true)
                {
                    if (enemies[topLeftEnemyNo].enemyState != Enemy.EnemyAIState.Destroyed)
                    {
                        moveLeft = false;

                        for (int k = 0; k < enemies.Count; k++)
                        {
                            enemies[k].UpdateDown(GraphicsDevice.Viewport);
                        }

                        lastY += GraphicsDevice.Viewport.Height / 30;

                        moveRight = true;
                    }

                    else
                    {
                        //if the enemy in the top left hand corner has been destroyed then we need to find a new next left most enemy
                        //Therefore first we check the left most column of enemies and see if any of them are still alive

                        //If no enemies in the first column alive we then look at the next column to try and find a living enemy there
                    }
                }

                //Keeps tabs on one of the most right most enemies so it knows when it has hit the right most wall of the game screen in order to know when to move down
                //and change direction
                if (enemies[topRightEnemyNo].enemyRect.X + enemies[topRightEnemyNo].enemyRect.Width >= GraphicsDevice.Viewport.Width && moveRight == true)
                {
                    moveRight = false;

                    for (int k = 0; k < enemies.Count; k++)
                    {
                        enemies[k].UpdateDown(GraphicsDevice.Viewport);
                    }

                    lastY = lastY + 20;

                    moveLeft = true;
                }

                for (int i = 0; i < enemies.Count; i++)
                {
                    if (gameTime.TotalGameTime - previousMovementTime > currentMovementTime)
                    {
                        // Reset our current time
                        previousMovementTime = gameTime.TotalGameTime;

                        for (int j = 0; j < enemies.Count; j++)
                        {
                            if (moveLeft == true)
                            {
                                if (enemies[j].enemyState != Enemy.EnemyAIState.BreakAwayLeft)
                                {
                                    enemies[j].UpdateLeft(GraphicsDevice.Viewport);
                                }
                            }

                            if (moveRight == true)
                            {
                                if (enemies[j].enemyState != Enemy.EnemyAIState.BreakAwayLeft)
                                {
                                    enemies[j].UpdateRight(GraphicsDevice.Viewport);
                                }
                            }
                        }

                        BreakAwayManager();
                    }

                    int remaining = EnemiesRemaining();

                    if (remaining == 0 && startOfRound == false)
                    {
                        endOfRound = true;
                        //Remove all of the enemies from the list                        
                        GarbageCollection();
                        NextRound();
                    }
                }
            }
        }  
       

        /// <summary>
        /// 
        /// </summary>
        /// <param name="enemyNumber"></param>
        public void UpdateBayesNetwork(int enemyNumber)
        {
            isWithinRadius(enemyNumber);
            isAbovePlayer(enemyNumber);
            isAboveBarrier(enemyNumber);
        }

        /// <summary>
        /// Determines whether or not an enemy is directly above the player 
        /// </summary>
        /// <param name="i">Enemy number</param>
        public void isAbovePlayer(int i)
        {
            //If the X coordinate at the centre of the enemy texture matches the X coordinate at the centre of the player texture then
            if (enemies[i].enemyRect.X + enemies[i].enemyRect.Width / 2 == player.playerRect.X + player.playerRect.Width / 2)
            {
                bayes.WithinRadius = false;
                bayes.AbovePlayer = true;
            }
        }

        /// <summary>
        /// Determines whether or not an enemy is directly above a barrier block 
        /// </summary>
        /// <param name="i">Barrier number</param>
        public void isAboveBarrier(int i)
        {
            for (int k = 0; k < barrier.Count - 1; k++)
            {
                //If the x coordinate at the centre of the enemy texture matches the X coordinate at the centre of the barrier block texture then
                if (enemies[i].enemyRect.X + (enemies[i].enemyRect.Width / 2) == barrier[k].barrierRect.X + (barrier[k].barrierRect.Width / 2))
                {
                    bayes.AboveBarrier = true;
                }
            }
        }

        /// <summary>
        /// Determines whether or not an enemy is within a specific radius of the player
        /// </summary>
        /// <param name="i">Enemy number</param>
        public void isWithinRadius(int i)
        {
            //If the X coordinate at the centre of the enemy texture is within a specific distance away from the enemy i.e. 20 pixels 
            //and is not currently directly above the player
            if (enemies[i].enemyRect.X + (enemies[i].enemyRect.Width / 2) >= player.playerRect.X - 20 && enemies[i].enemyRect.X + (enemies[i].enemyRect.Width / 2) <= player.playerRect.X + player.playerRect.Width + 20)
            {
                bayes.WithinRadius = true;
            }
        }

        /// <summary>
        /// Determines how many enemies are left alive
        /// </summary>
        /// <returns>The number of enemies remaining</returns>
        public int EnemiesRemaining()
        {
            int remaining = 0;

            for (int i = 0; i < enemies.Count; i++)
            {
                if (enemies[i].enemyState != Enemy.EnemyAIState.Destroyed)
                {
                    remaining++;
                }
            }

            return remaining;
        }

        /// <summary>
        /// Finds the left most enemy the group
        /// </summary>
        /// <param name="i">Enemy number</param>
        /// <returns>The number of the left most enemy</returns>
        public int FindLeftMostEnemy(int i)
        {
            int leftMost = 0, temp = i, count = 1;
            Boolean found = false;

            for (int z = 0; z < 5; z++)
            {
                if (temp >= 10)
                {
                    if (enemies[i - 10].enemyState != Enemy.EnemyAIState.Destroyed)
                    {
                        temp = temp - 10;
                    }
                }
            }

            //Keep looping through enemies until a match is found
            while (found == false)
            {
                for (int k = 0; k < 5; k++)
                {
                    //Go through the enemies column by column to find the left most enemy                    
                    if (temp < enemies.Count - 10 && temp >= 0)
                    {
                        if (enemies[temp].enemyState != Enemy.EnemyAIState.Destroyed)
                        {
                            leftMost = temp;
                            found = true;
                            break;
                        }
                    }

                    //jump to the next enemy down in the column
                    temp = temp + 10;
                }

                temp = i + count;
                count++;
            }

            return leftMost;
        }

        /// <summary>
        /// Finds the right most enemy the group
        /// </summary>
        /// <param name="i">Enemy number</param>
        /// <returns>The number of the right most enemy</returns>
        public int FindRightMostEnemy(int i)
        {
            int rightMost = 0, temp = i, count = 1;
            Boolean found = false;

            for (int z = 0; z < 5; z++)
            {
                if (temp >= 10)
                {
                    if (enemies[i - 10].enemyState != Enemy.EnemyAIState.Destroyed)
                    {
                        temp = temp - 10;
                    }
                }
            }

            while (found == false)
            {                
                //will probably need to make this a recursive function that counts how many columns its has searched for
                for (int j = 0; j < 5; j++)
                {
                    //Go through the enemies column by column to find the right most enemy 
                    if (temp < enemies.Count - 10 && temp >= 0)
                    {
                        if (enemies[temp].enemyState != Enemy.EnemyAIState.Destroyed)
                        {
                            rightMost = temp;
                            found = true;
                            break;
                        }

                        temp = temp + 10;
                    }
                }

                temp = i - count;
                count++;
            }

            return rightMost;
        }      

        /// <summary>
        /// Updates which enemy is next to fire upon an enemy being destroyed 
        /// </summary>
        /// <param name="i">Enemy number</param>
        public void UpdateFiringEnemiesList(int i)
        {
            //Retrieve the values in the jagged array to access the pointer and
            //to which the next enemy in the list is to fire
            float tempx = 0, tempy = 0;
            int? nextFire = 0, tempPointer = 0;

            bool corrected = false;
            int count = 1;
            
            tempx = enemies[i].Number.X;
            tempy = enemies[i].Number.Y;

            if (enemies[i].prevState == Enemy.previousState.HasLineofSight)
            {
                if (tempy != null && tempx != null)
                {
                    nextFire = enemyBlock[(int)tempx][(int)tempy];

                    if (nextFire != null)
                    {
                        enemies[(int)nextFire].changeState(idle, hasLineofSight);
                    }

                    enemyBlock[(int)tempx][(int)tempy] = null;                   
                }

                else
                {
                    nextFire = enemyBlock[(int)tempx][(int)tempy];

                    if (tempx != null)
                    {
                        enemies[(int)nextFire].changeState(idle, hasLineofSight);
                    }

                    enemyBlock[(int)tempx][(int)tempy] = null;             
                }         
            }

            else
            {               
                tempx = enemies[i].Number.X;
                tempy = enemies[i].Number.Y;
               
                tempPointer = enemyBlock[(int)tempx][(int)tempy];
              
                enemyBlock[(int)tempx][(int)tempy] = null;

                while (corrected == false)
                {
                    if (enemyBlock[(int)tempx + count][(int)tempy] != null)
                    {
                        enemyBlock[(int)tempx + count][(int)tempy] = tempPointer;
                        corrected = true;
                        break;
                    }

                    else
                    {
                        count++;
                    }
                }             
            }
        }

        /// <summary>
        /// Adds a full pack of enemies to the game screen
        /// </summary>
        /// <param name="yPos">Y coordinate of the first enemy in the top left hand corner</param>
        public void AddEnemy(int yPos)
        {
            int enemyCount = 0, row = 0;
            Vector2 arrayPos = new Vector2(row, enemyCount);

            topRightCount = 9;

            Vector2 enemyPos = new Vector2();

            enemyPos.X = (GraphicsDevice.Viewport.Width / 4);
            enemyPos.Y = yPos;

            for (int i = 0; i <= 9; i++)
            {
                //Create the top row of enemies
                badGuy2 = new BadGuy2();

                badGuy2.Initialize(enemyTexture1, levelNo, enemyPos, GraphicsDevice.Viewport, arrayPos);

                enemyMovement = badGuy2.enemySpeed;

                enemies.Add(badGuy2);
                enemyCount++;
                arrayPos.Y++;

                enemyPos.X = enemyPos.X + (badGuy2.enemyRect.Width * 2);

                if (i == 9)
                {
                    arrayPos.X++;
                    arrayPos.Y = 0;

                    enemyPos.X = enemyPos.X - ((badGuy2.enemyRect.Width * 2) * 10);
                    enemyPos.Y = enemyPos.Y + badGuy2.enemyRect.Height + (badGuy2.enemyRect.Height / 2);
                }
            }

            for (int i = 0; i <= 19; i++)
            {
                //Create the top row of enemies
                badGuy1 = new BadGuy1();

                badGuy1.Initialize(enemyTexture2, levelNo, enemyPos, GraphicsDevice.Viewport, arrayPos);
                enemyMovement = badGuy1.enemySpeed;

                enemies.Add(badGuy1);
                enemyCount++;
                arrayPos.Y++;

                enemyPos.X = enemyPos.X + (badGuy1.enemyRect.Width * 2);

                if (i == 9 || i == 19)
                {
                    arrayPos.X++;
                    arrayPos.Y = 0;

                    enemyPos.X = enemyPos.X - ((badGuy1.enemyRect.Width * 2) * 10);
                    enemyPos.Y = enemyPos.Y + badGuy1.enemyRect.Height + (badGuy1.enemyRect.Height / 2);
                }
            }

            for (int i = 0; i <= 19; i++)
            {
                //Create the top row of enemies
                badGuy = new BadGuy();

                badGuy.Initialize(enemyTexture3, levelNo, enemyPos, GraphicsDevice.Viewport, arrayPos);
                enemyMovement = badGuy.enemySpeed;

                enemies.Add(badGuy);
                enemyCount++;
                arrayPos.Y++;

                enemyPos.X = enemyPos.X + (badGuy.enemyRect.Width * 2);

                if (i == 9 || i == 19)
                {
                    arrayPos.X++;
                    arrayPos.Y = 0;
                    enemyPos.X = enemyPos.X - ((badGuy.enemyRect.Width * 2) * 10);
                    enemyPos.Y = enemyPos.Y + badGuy.enemyRect.Height + (badGuy.enemyRect.Height / 2);
                }
            }
        }

        /// <summary>
        /// Adds a rare enemy to the game 
        /// </summary>
        public void AddRareEnemy()
        {
            rareEnemy = new RareEnemy();
            Vector2 rareEnemyPos;

            rareEnemy.Initialize(enemyTexture4, GraphicsDevice.Viewport);

            if (rareEnemy.Direction == 1)
            {
                rareEnemyPos.X = 10;
                rareEnemyPos.Y = 10;
            }

            if (rareEnemy.Direction == 2)
            {
                rareEnemyPos.X = GraphicsDevice.Viewport.Width;
                rareEnemyPos.Y = 10;
            }

            rareEnemyActive = true;
        }
        

        /// <summary>
        /// Updates the position of the enemy based on its direction 
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateRareEnemy(GameTime gameTime)
        {
            if (endOfRound == false)
            {
                if (rareEnemy.Direction == 1)
                {
                    rareEnemy.UpdateRight();

                    if (rareEnemy.enemyRect.X >= GraphicsDevice.Viewport.Width)
                    {
                        rareEnemy.Active = false;
                        rareEnemyActive = false;
                    }
                }

                else if (rareEnemy.Direction == 2)
                {
                    rareEnemy.UpdateLeft();

                    if (rareEnemy.enemyRect.X <= 0)
                    {
                        rareEnemy.Active = false;
                        rareEnemyActive = false;
                    }

                }
            }
        }

        /// <summary>
        /// Adds a full set of 4 barriers to the game screen
        /// </summary>
        /// <param name="position"></param>
        public void AddBarriers(Vector2 position)
        {
            for (int i = 0; i <= 7; i++)
            {
                Barrier barrierBlock = new Barrier();

                if (i != 2 || i != 5)
                {
                    //Initialize the barrier block
                    barrierBlock.Initialize(GraphicsDevice.Viewport, barrierTexture, position);

                    if (i <= 1)
                    {
                        //Set the y position of  the next barrier to sit on top of the one just created
                        position.Y = position.Y - barrierBlock.barrierRect.Height;
                    }

                    if (i == 3 || i == 4)
                    {
                        position.X = position.X + barrierBlock.barrierRect.Width;
                    }

                    if (i == 6)
                    {
                        //Set the y position of the next barrier to sit beneath of the one just created
                        position.Y = position.Y + barrierBlock.barrierRect.Height;
                    }
                }

                if (i == 2)
                {
                    //Initialize the left corner barrier block
                    barrierBlock.Initialize(GraphicsDevice.Viewport, barrierLeftCornerTexture, position);
                    position.X = position.X + barrierBlock.barrierRect.Width;
                }

                if (i == 5)
                {
                    //Initialize the right corner barrier block
                    barrierBlock.Initialize(GraphicsDevice.Viewport, barrierRightCornerTexture, position);
                    position.Y = position.Y + barrierBlock.barrierRect.Height;
                }

                barrier.Add(barrierBlock);
            }

            while (barrierCount < 3)
            {
                barrierCount++;
                position.X = position.X + eighthOfScreen - (barrierBlock.barrierRect.Width * 2);

                AddBarriers(position);
            }
        }

        /// <summary>
        /// Update the barriers 
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateBarrier(GameTime gameTime)
        {
            //Update barriers
            for (int i = 0; i < barrier.Count; i++)
            {
                if (barrier[i].Active == false)
                {
                    barrier.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// Update the lasers fired by the player 
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateLaser(GameTime gameTime)
        {
            //Update lasers
            for (int i = lasers.Count - 1; i >= 0; i--)
            {
                lasers[i].Update();

                if (lasers[i].Active == false)
                {
                    lasers.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// Add a laser fired by the player to the game screen
        /// </summary>
        /// <param name="position">Position of the laser relative to the players position</param>
        public void AddLaser(Vector2 position)
        {
            Laser laser = new Laser();
            laser.Initialize(GraphicsDevice.Viewport, laserTexture, position);
            lasers.Add(laser);
        }

        /// <summary>
        /// Update the laser fired by an enemy
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateEnemyLaser(GameTime gameTime)
        {
            //This list will store the value of the next enemy ship in sequence that should start firing
            //These values will need to be looped in the update enemy method so the list below will probably need to made as a global variable so it can be accessed by the methods

            for (int i = enemyLasers.Count - 1; i >= 0; i--)
            {
                enemyLasers[i].Update();

                if (enemyLasers[i].Active == false)
                {
                    enemyLasers.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// Add a laser fired by an enemy to the game screeen
        /// </summary>
        /// <param name="position">Position of the laser relative to the enemies position</param>
        public void AddEnemyLaser(Vector2 position)
        {
            EnemyLaser enemyLaser = new EnemyLaser();
            enemyLaser.Initialize(GraphicsDevice.Viewport, enemyLaserTexture, position);
            enemyLasers.Add(enemyLaser);
        }

        /// <summary>
        /// Update any collisions that occur in the game 
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateCollision(GameTime gameTime)
        {
            //Creates a rectangle to be used to go around the lasers
            Rectangle playerLaserRectangle;

            //Creates a rectangle to be used to go around the enemies
            Rectangle enemyRectangle;

            //Creates a rectangle to be used to go around the enemy lasers
            Rectangle enemyLaserRectangle;

            //Creates a rectangle to be used to around the player 
            Rectangle playerRectangle;

            //Creates a rectangle to be used around the mystery enemy
            Rectangle mysteryEnemyRectangle;

            //Creates a rectangle to be used to go around the barrier blocks
            Rectangle barrierRectangle;

            //Creates a rectangle to be used to go around the shield power up
            Rectangle shieldPowerUpRectangle;

            //Creates a rectangle to be used to go around the movement power up
            Rectangle movementPowerUpRectangle;

            //Creates a rectangle to be used to go around the double laser power up
            Rectangle doublePowerUpRectangle;

            //Creates a rectangle to be used to go around the shield
            Rectangle shieldRectangle;

            for (int i = 0; i < powerUps.Count; i++)
            {
                playerRectangle = new Rectangle((int)player.playerRect.X, (int)player.playerRect.Y, player.playerRect.Height, player.playerRect.Width);

                shieldPowerUpRectangle = new Rectangle((int)powerUps[i].powerUpShieldRect.X, (int)powerUps[i].powerUpShieldRect.Y, powerUps[i].powerUpShieldRect.Width, powerUps[i].powerUpShieldRect.Height);

                movementPowerUpRectangle = new Rectangle((int)powerUps[i].powerUpMovementRect.X, (int)powerUps[i].powerUpMovementRect.Y, powerUps[i].powerUpMovementRect.Width, powerUps[i].powerUpMovementRect.Height);

                doublePowerUpRectangle = new Rectangle((int)powerUps[i].powerUpDoubleRect.X, (int)powerUps[i].powerUpDoubleRect.Y, powerUps[i].powerUpDoubleRect.Width, powerUps[i].powerUpDoubleRect.Height);


                if (shieldPowerUpRectangle.Intersects(playerRectangle))
                {
                    powerUps[i].dropPowerActive = false;
                    player.shields = true;
                    shieldActivationTime = gameTime.TotalGameTime;
                }

                if (movementPowerUpRectangle.Intersects(playerRectangle))
                {
                    powerUps[i].dropPowerActive = false;
                    IncreaseMovement();
                    movementActivationTime = gameTime.TotalGameTime;
                }

                if (doublePowerUpRectangle.Intersects(playerRectangle))
                {
                    powerUps[i].dropPowerActive = false;
                    player.dualLasers = true;
                    dualLasersActivationTime = gameTime.TotalGameTime;
                }
            }

            //Creates a rectangle to go around the enemy lasers
            //Is in the loop....            

            for (int h = 0; h < enemies.Count; h++)
            {
                enemyRectangle = new Rectangle((int)enemies[h].enemyRect.X, (int)enemies[h].enemyRect.Y, enemies[h].enemyRect.Height, enemies[h].enemyRect.Width);
                playerRectangle = new Rectangle((int)player.playerRect.X, (int)player.playerRect.Y, player.playerRect.Height, player.playerRect.Width);

                if (enemyRectangle.Intersects(playerRectangle) && (enemies[h].enemyState == Enemy.EnemyAIState.BreakAwayLeft || enemies[h].enemyState == Enemy.EnemyAIState.BreakAwayRight))
                {
                    player.Active = false;
                    int state = getCurrentState(h);
                    enemies[h].changeState(state, destroyed);

                    UpdateFiringEnemiesList(h);
                    UpdateBreakAwayPointers(h);
                }

                else if (enemyRectangle.Intersects(playerRectangle))
                {
                    player.Lives = 1;
                    player.Active = false;
                }
            }

            for (int i = 0; i < lasers.Count; i++)
            {
                //Check to see if any of the players lasers collide with the mystery enemy 
                playerLaserRectangle = new Rectangle((int)lasers[i].laserRect.X, (int)lasers[i].laserRect.Y, lasers[i].laserRect.Width, lasers[i].laserRect.Height);
                mysteryEnemyRectangle = new Rectangle((int)rareEnemy.enemyRect.X, (int)rareEnemy.enemyRect.Y, (int)rareEnemy.enemyRect.Width, (int)rareEnemy.enemyRect.Height);

                if (playerLaserRectangle.Intersects(mysteryEnemyRectangle))
                {
                    rareEnemy.Active = false;
                    rareEnemyActive = false;
                    player.Score = player.Score + rareEnemy.Value;
                }

                //Check to see if a player laser collides with a rectangle
                for (int k = 0; k < barrier.Count; k++)
                {
                    barrierRectangle = new Rectangle((int)barrier[k].barrierRect.X, (int)barrier[k].barrierRect.Y, barrier[k].barrierRect.Height, barrier[k].barrierRect.Width);

                    if (playerLaserRectangle.Intersects(barrierRectangle))
                    {
                        barrier[k].barrierHealth--;

                        barrier[k].Update(barrier[k].barrierHealth);

                        lasers[i].Active = false;
                        break;
                    }
                }

                for (int j = 0; j < enemies.Count; j++)
                {
                    if (enemies[j].enemyState != Enemy.EnemyAIState.Destroyed)
                    {
                        enemyRectangle = new Rectangle((int)enemies[j].enemyRect.X, (int)enemies[j].enemyRect.Y, enemies[j].enemyRect.Height, enemies[j].enemyRect.Width);

                        //If the laser rectangle intersects the enemy rectangle then
                        if (playerLaserRectangle.Intersects(enemyRectangle))
                        {
                            //Deactivate said laser
                            lasers[i].Active = false;

                            int state = getCurrentState(j);

                            if (state != canBreakAwayLeft || state != canBreakAwayLeft || state != breakAwayLeft || state != breakAwayRight || state != cantBreakLeft || state != cantBreakRight)
                            {
                                enemies[j].changeState(state, destroyed);
                                UpdateFiringEnemiesList(j);
                            }

                            else
                            {
                                enemies[j].changeState(state, destroyed);
                                UpdateFiringEnemiesList(j);
                                UpdateBreakAwayPointers(j);
                            }

                            int remaining = EnemiesRemaining();

                            if (j == topLeftEnemyNo && remaining != 0)
                            {
                                topLeftEnemyNo = FindLeftMostEnemy(j);

                            }

                            if (j == topRightEnemyNo && remaining != 0)
                            {
                                topRightEnemyNo = FindRightMostEnemy(j);
                            }

                            //Add the value of the enemy to the player score
                            player.Score = player.Score + enemies[j].Value;

                            if (modern == true)
                            {
                                //Check to see if that enemy will drop a powerup
                                int probability = random.Next(0, 100);

                                //20% chance of a power up dropping
                                if (probability >= 20)
                                {
                                    AddPowerUp(new Vector2(enemies[j].enemyRect.X, enemies[j].enemyRect.Y));
                                }
                            }
                        }
                    }
                }
            }

            //Update any collisions between the enemy lasers and the players lasers
            for (int l = 0; l < enemyLasers.Count; l++)
            {
                enemyLaserRectangle = new Rectangle((int)enemyLasers[l].enemyLaserRect.X - enemyLasers[l].enemyLaserRect.Width / 2, (int)enemyLasers[l].enemyLaserRect.Y - enemyLasers[l].enemyLaserRect.Height / 2, enemyLasers[l].enemyLaserRect.Width, enemyLasers[l].enemyLaserRect.Height);
                playerRectangle = new Rectangle((int)player.playerRect.X, (int)player.playerRect.Y, player.playerRect.Height, player.playerRect.Width);


                if (enemyLaserRectangle.Intersects(playerRectangle))
                {
                    //Player has been killed, muah, muahaha, MUAHAHAHA
                    player.Active = false;
                }

                //Check to see if enemy lasers collide with barrier
                //Rectangle barrierRectangle;

                for (int k = 0; k < barrier.Count; k++)
                {
                    barrierRectangle = new Rectangle((int)barrier[k].barrierRect.X, (int)barrier[k].barrierRect.Y, barrier[k].barrierRect.Height, barrier[k].barrierRect.Width);

                    if (enemyLaserRectangle.Intersects(barrierRectangle))
                    {
                        barrier[k].barrierHealth--;

                        barrier[k].Update(barrier[k].barrierHealth);

                        enemyLasers[l].Active = false;
                    }
                }
            }

            for (int m = 0; m < enemies.Count; m++)
            {
                int state = getCurrentState(m);

                enemyRectangle = new Rectangle((int)enemies[m].enemyRect.X, (int)enemies[m].enemyRect.Y, enemies[m].enemyRect.Height, enemies[m].enemyRect.Width);

                for (int k = 0; k < barrier.Count; k++)
                {
                    barrierRectangle = new Rectangle((int)barrier[k].barrierRect.X, (int)barrier[k].barrierRect.Y, barrier[k].barrierRect.Height, barrier[k].barrierRect.Width);

                    if (enemies[m].enemyState != Enemy.EnemyAIState.Destroyed)
                    {
                        if (enemyRectangle.Intersects(barrierRectangle))
                        {

                            if (state != breakAwayLeft || state != breakAwayRight)
                            {
                                enemies[m].changeState(state, destroyed);
                                UpdateFiringEnemiesList(m);

                                barrier[k].barrierHealth = 0;
                                barrier[k].Update(barrier[k].barrierHealth);
                            }

                            else
                            {
                                enemies[m].changeState(state, destroyed);
                                UpdateFiringEnemiesList(m);
                                UpdateBreakAwayPointers(m);

                                barrier[k].barrierHealth--;
                                barrier[k].Update(barrier[k].barrierHealth);
                            }

                            break;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Start the next round of the game
        /// </summary>
        public void NextRound()
        {
            endOfRound = false;
            levelNo++;
            player.Round++;            
            startOfRound = true;
            allDead = false;

            if (modern == true)
            {
                bayes.fireTrue *= bayes.fireTrue;
            }
        }

        /// <summary>
        /// Respawn the player upon death
        /// </summary>
        public void PlayerRespwan()
        {
            condition = true;
            AddPlayer();
        }

        /// <summary>
        /// Exit the game 
        /// </summary>
        /// <param name="exit">A boolean depicting whether exit = true/false</param>
        public void Quit(Boolean exit)
        {
            if (exit == true)
            {
                this.Exit();
            }
        }

        /// <summary>
        /// Return the user to the main menu
        /// </summary>
        public void ReturnToMenu()
        {
            Play = false;
            Instructions = false;
            Paused = false;
            Controls = false;
            subMenu = false;

            Menu = true;
        }

        public void NewGame()
        {
            Play = false;
            Instructions = false;
            Paused = false;
            Controls = false;
            subMenu = false;

            Menu = true;

            menu.Initialize(GraphicsDevice.Viewport);

            startOfRound = true;
            levelNo = 1;

            GarbageCollection();
        }

        /// <summary>
        /// Removes any outstanding objects on the game screen from lists and arrays to free up memory
        /// </summary>
        public void GarbageCollection()
        {
            //The deletion of enemy objects has to be done back to front as the array is dynamic            
            for (int i = enemies.Count - 1; i >= 0; i--)
            {
                //De-reference the object so it is picked up by the garbage collector 
                enemies[i] = null;
                enemies.RemoveAt(i);
            }


            for (int j = lasers.Count - 1; j >= 0; j--)
            {
                //De-reference the object so it is picked up by the garbage collector 
                lasers[j] = null;
                lasers.RemoveAt(j);
            }

            for (int k = enemyLasers.Count - 1; k >= 0; k--)
            {
                //De-reference the object so it is picked up by the garbage collector 
                enemyLasers[k] = null;
                enemyLasers.RemoveAt(k);
            }
        }

        public List<Vector2> GenerateCurveRight(Vector2 enemyPos, Viewport viewport)
        {
            path = new List<Vector2>();
            Vector2 PlotPoint;

            Vector2 control1 = Vector2.Zero, control2 = Vector2.Zero;
            Vector2 key2 = Vector2.Zero;

            //set the key point to the bottom right hand corner of the screen
            key2.X = 0;
            key2.Y =  viewport.Height;

            control1.X = enemyPos.X + viewport.Width / 5;
            control1.Y = enemyPos.Y + viewport.Height / 10;

            control2.X = control1.X;
            control2.Y = control1.Y + viewport.Height / 10;

            for (float t = 0; t <= 1.0f; t += 0.01f)
            {
                PlotPoint = GetCurveCoordinates(t, enemyPos, control1, control2, key2);
                path.Add(PlotPoint);
            }

            return path;
        }

        public List<Vector2> GenerateCurveLeft(Vector2 enemyPos, Viewport viewport)
        {
            path = new List<Vector2>();
            Vector2 PlotPoint;

            Vector2 control1 = Vector2.Zero, control2 = Vector2.Zero;
            Vector2 key2 = Vector2.Zero;

            //set the key point to the bottom right hand corner of the screen
            key2.X = GraphicsDevice.Viewport.Width;
            key2.Y = GraphicsDevice.Viewport.Height;

            control1.X = enemyPos.X - viewport.Width / 5;
            control1.Y = enemyPos.Y + viewport.Height / 10;

            control2.X = control1.X;
            control2.Y = control1.Y + viewport.Height / 10;

            for (float t = 0; t <= 1.0f; t += 0.1f)
            {
                PlotPoint = GetCurveCoordinates(t, enemyPos, control1, control2, key2);
                path.Add(PlotPoint);
            }

            return path;
        }

        /// <summary>
        /// Generate a bezier curve for the enemy to follow when breaking away
        /// </summary>
        /// <param name="t">time interval</param>
        /// <param name="p0">key point 1</param>
        /// <param name="p1">control point 1</param>
        /// <param name="p2">key point 2</param>
        /// <param name="p3">control point 2</param>
        /// <returns>bezier point</returns>        
        public Vector2 GetCurveCoordinates(double t, Vector2 p0, Vector2 p1, Vector2 p2, Vector2 p3)
        {
            Vector2 b;
            b.X = (float)(Math.Pow((1 - t), 3) * p0.X + 3 * Math.Pow((1 - t), 2) * t * p1.X + 3 * (1 - t) * Math.Pow(t, 2) * p2.X + Math.Pow(t, 3) * p3.X);
            b.Y = (float)(Math.Pow((1 - t), 3) * p0.Y + 3 * Math.Pow((1 - t), 2) * t * p1.Y + 3 * (1 - t) * Math.Pow(t, 2) * p2.Y + Math.Pow(t, 3) * p3.Y);
            
            return new Vector2(b.X, b.Y);
        }

        /// <summary>
        /// Update the position of the power up
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdatePowerUp(GameTime gameTime)
        {
            for (int i = 0; i < powerUps.Count; i++)
            {
                powerUps[i].UpdatePowerUp(powerUps[i].Type);

                if (powerUps[i].dropPowerActive == false)
                {
                    powerUps[i] = null;
                    powerUps.RemoveAt(i);
                }
            }
        }

        public void IncreaseMovement()
        {
            player.playerMoveSpeed += (player.playerMoveSpeed / 2);
        }

        public void UpdateTimers(GameTime gameTime)
        {
            TimeSpan powerUpDuration = TimeSpan.FromSeconds(5f);
            if (player.shields == true)
            {
                if (gameTime.TotalGameTime - shieldActivationTime > powerUpDuration)
                {
                    player.shields = false;
                }
            }

            if (player.dualLasers == true)
            {
                if (gameTime.TotalGameTime - dualLasersActivationTime > powerUpDuration)
                {
                    player.dualLasers = false;
                }
            }

            if (player.movement == true)
            {
                if (gameTime.TotalGameTime - movementActivationTime > powerUpDuration)
                {
                    player.movement = false;
                }
            }
        }

        /// <summary>
        /// Add a power up to the game screen 
        /// </summary>
        /// <param name="value">Number of points obtained if powerup is collected</param>
        /// <param name="pos">Position of the power up relative to the position of the destroyed enemy</param>
        public void AddPowerUp(Vector2 pos)
        {
            int probability1 = random.Next(0, 100);

            if (probability1 <= 40)
            {
                PowerUp powerz = new PowerUp();
                powerz.InitialisePowerUps(powerUpShield, powerUpMovement, powerUpDouble, GraphicsDevice.Viewport, pos, shieldPower);
                powerUps.Add(powerz);
            }

            else if (probability1 > 40 && probability1 <= 80)
            {
                PowerUp powerz = new PowerUp();
                powerz.InitialisePowerUps(powerUpShield, powerUpMovement, powerUpDouble, GraphicsDevice.Viewport, pos, movementPower);
                powerUps.Add(powerz);
            }

            else if (probability1 > 80 && probability1 <= 100)
            {
                PowerUp powerz = new PowerUp();
                powerz.InitialisePowerUps(powerUpShield, powerUpMovement, powerUpDouble, GraphicsDevice.Viewport, pos, doubleLaser);
                powerUps.Add(powerz);
            }           
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            // Start drawing
            spriteBatch.Begin();

            if (Play == true)
            {                
                menu.DrawBackground(spriteBatch, background, GraphicsDevice.Viewport);

                playScreen.DrawBorder(spriteBatch);

                playScreen.DrawLives(spriteBatch, font, player.Lives);

                playScreen.DrawRound(spriteBatch, font, player.Round);

                playScreen.DrawScore(spriteBatch, font, player.Score);

                if (player.Active == true)
                {                   
                    // Draw the Player
                    if (isMovingLeft == false && isMovingRight == false)
                    {
                        if (player.shields == false)
                        {
                            player.Draw(spriteBatch);
                        }

                        else
                        {
                            player.DrawShield(spriteBatch);
                        }
                    }

                    if (isMovingLeft == true)
                    {
                        if (player.shields == false)
                        {
                            player.DrawLeft(spriteBatch);
                        }

                        else
                        {
                            player.DrawLeftShield(spriteBatch);
                        }
                    }

                    if (isMovingRight == true)
                    {
                        if (player.shields == false)
                        {
                            player.DrawRight(spriteBatch);
                        }

                        else
                        {
                            player.DrawRightShield(spriteBatch);
                        }
                    }
                }


                if (rareEnemyActive == true)
                {
                    //Draw the rare enemy
                    rareEnemy.Draw(spriteBatch);
                }

                //Draw the power ups 
                for (int i = 0; i < powerUps.Count; i++)
                {
                    powerUps[i].DrawPowerUp(spriteBatch, powerUps[i].Type);                    
                }

                //Draw the enemies 
                for (int i = 0; i < enemies.Count; i++)
                {
                    if (enemies[i].enemyState != Enemy.EnemyAIState.Destroyed)
                    {
                        enemies[i].Draw(spriteBatch);
                    }
                }

                //Draw the lasers
                for (int i = 0; i < lasers.Count; i++)
                {
                    lasers[i].Draw(spriteBatch);
                }

                //Draw the barriers
                for (int i = 0; i < barrier.Count; i++)
                {
                    barrier[i].Draw(spriteBatch);
                }

                //Draw the enemy lasers
                for (int i = 0; i < enemyLasers.Count; i++)
                {
                    enemyLasers[i].Draw(spriteBatch);
                }
            }

            if (SplashScreen == true)
            {
                splash.DrawSplash(spriteBatch);
            }

            //Write main menu text to the screen
            if (Menu == true)
            {
                menu.DrawBackground(spriteBatch, background, GraphicsDevice.Viewport);
                menu.Draw(spriteBatch, font, font1, menu.MenuPosition, laserTexture);
                menu.DrawNavigation(spriteBatch, font1);

                if (subMenu == true)
                {
                    menu.DrawSubMenu(spriteBatch, font, font, menu.SubMenuPosition, laserTexture);
                }
            }

            if (Instructions == true)
            {
                menu.DrawBackground(spriteBatch, background, GraphicsDevice.Viewport);
                instructions.Draw(spriteBatch, font, font1);
                instructions.DrawImages(spriteBatch);
            }            

            if (Paused == true)
            {
                menu.DrawBackground(spriteBatch, background, GraphicsDevice.Viewport);
                paused.DrawPausedScreen(spriteBatch, font1);
            }

            if (Controls == true)
            {
                controlScreen.DrawScreen(spriteBatch, font1);
            }

            //Stop drawing
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
