using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Collections.Generic;

namespace SpaceInvadersPhone
{
    public class Instructions
    {
        //Used to store the positions of the main title and the values of each of the enemies
        public Vector2 titlePos, subTitle1Pos, subTitle2Pos, value1Pos, value2Pos, returnToMenuPos;

        //Used to store the position of the first enemy image and first power up image
        public Vector2 enemyImagePos, powerUpImagePos;

        //Need a variable to store the positions of the images to reflect the enemies

        //Colours used in the menu
        public Color colourRed, colourWhite;

        //Store the text for the main title as a string
        public string title, subTitle1, subTitle2, value1, shieldPowerUp, dualLaserPowerUp, movementSpeedPowerUp;

        //Store the textures (images) of the enemies
        Texture2D enemyTexture, enemyTexture2, enemyTexture3, enemyTexture4, shieldTexture, movementTexture, dualLaserTexture, RectTex;

        //Creates a list of textures
        List<Texture2D> textures = new List<Texture2D>();

        //Rectangles with an aspect ratio to contain the enemy image
        public Rectangle rect, returnToMenuRect;

        //Creates a list for all of the rectangles to be drawn to screen
        List<Rectangle> Rectangles = new List<Rectangle>();        

        List<Vector2> textPos = new List<Vector2>();

        Viewport view;

        // Get the width of the enemy ship
        public int Width
        {
            get { return enemyTexture.Width; }
        }

        // Get the height of the player ship
        public int Height
        {
            get { return enemyTexture.Height; }
        }

        public void Initialise(Viewport viewport)
        {           
            view = viewport;
            title = "Instructions";
            subTitle1 = "Enemies";
            subTitle2 = "Power Ups - Shoot or collide with to unlock!";
            value1 = "????";
            shieldPowerUp = "Shield power up";
            dualLaserPowerUp = "Dual lasers power up";
            movementSpeedPowerUp = "Movement speed power up";

            //Initialise colours for menu fonts
            colourRed = new Color(255, 0, 0);
            colourWhite = new Color(255, 255, 255);            

            titlePos.X = view.Width / 3;
            titlePos.Y = view.Height / 50;

            subTitle1Pos.X = view.Width / 5;
            subTitle1Pos.Y = view.Height / 8;

            subTitle2Pos.X = subTitle1Pos.X + view.Width / 3;
            subTitle2Pos.Y = view.Height / 8;

            //Initialise the positions of the values of the enemies
            value1Pos.X = view.Width / 4;
            value1Pos.Y = view.Height / 7;

            //Initialise the position of the main body of text 
            value2Pos.X = view.Width / 12;
            value2Pos.Y = view.Height / (float)1.25;

            //Initialise the position of the return to menu button
            returnToMenuPos.X = view.Width / (float)1.4;
            returnToMenuPos.Y = view.Height / (float)1.1;
            
            //Initialise the position of the first image
            enemyImagePos.X = view.Width / 9;
            enemyImagePos.Y = view.Height / 7;

            powerUpImagePos.X = enemyImagePos.X + view.Width / 3;
            powerUpImagePos.Y = view.Height / 7;


        }

        public void initialiseTextPositions()
        {
            Vector2 temp = value1Pos;

            for (int i = 0; i < 8; i++)
            {
                textPos.Add(value1Pos);

                value1Pos.Y = value1Pos.Y + view.Height / 7;

                if (i == 3)
                {
                    value1Pos.X += view.Width / 3;
                    value1Pos.Y = temp.Y;
                }
            }
        }

        public void initialiseRectangles(Texture2D enemyTex, Texture2D enemyTex2, Texture2D enemyTex3, Texture2D enemyTex4, Texture2D shieldTex, Texture2D movementTex, Texture2D dualTex, Texture2D rectTex)
        {
            enemyTexture = enemyTex;
            enemyTexture2 = enemyTex2;
            enemyTexture3 = enemyTex3;
            enemyTexture4 = enemyTex4;
            RectTex = rectTex;

            shieldTexture = shieldTex;
            movementTexture = movementTex;
            dualLaserTexture = dualTex;

            textures.Add(enemyTex4);
            textures.Add(enemyTexture);
            textures.Add(enemyTexture2);
            textures.Add(enemyTexture3);

            textures.Add(shieldTexture);
            textures.Add(movementTexture);
            textures.Add(dualLaserTexture);

            for (int h = 0; h < 4; h++)
            {
                createRectangle(textures[h], enemyImagePos);
                enemyImagePos.Y += view.Height / 7;
            }

            for (int i = 0; i < textures.Count; i++)
            {
                if (i > 3)
                {
                    createRectangle(textures[i], powerUpImagePos);
                    powerUpImagePos.Y += view.Height / 7;
                }
            }

            initialiseTextPositions();
        }

        public void createRectangle(Texture2D texture, Vector2 position)
        {
            float AspectRatio = (float)texture.Width / texture.Height;

            float Width = (float)view.Width / 15;

            float Height = Width / AspectRatio;


            rect = new Rectangle((int)position.X, (int)position.Y, (int)Width, (int)Height);
            Rectangles.Add(rect);
        }

        public void Draw(SpriteBatch spriteBatch, SpriteFont font, SpriteFont font1)
        {
            spriteBatch.DrawString(font1, title, titlePos, colourRed);
            
            for (int i = 0; i < textPos.Count; i++)
            {
                switch (i)
                {
                    case 0:
                        spriteBatch.DrawString(font, value1, textPos[i], colourWhite);
                        break;

                    case 1:
                        spriteBatch.DrawString(font, "40", textPos[i], colourWhite);
                        break;

                    case 2:
                        spriteBatch.DrawString(font, "20", textPos[i], colourWhite);
                        break;

                    case 3:
                        spriteBatch.DrawString(font, "10", textPos[i], colourWhite);
                        break;

                    case 4:
                        spriteBatch.DrawString(font, "Shield power up", textPos[i], colourWhite);
                        break;

                    case 5:
                        spriteBatch.DrawString(font, "Dual lasers power up", textPos[i], colourWhite);
                        break;

                    case 6:
                        spriteBatch.DrawString(font, "Movement power up", textPos[i], colourWhite);
                        break;

                    case 7:
                        spriteBatch.DrawString(font, "(Modern mode only)", textPos[i], colourWhite);
                        break;
                }
            }

            spriteBatch.DrawString(font, "Destroy as many enemies as possible without dying \n to protect your planet!", value2Pos, colourWhite);
            spriteBatch.DrawString(font, "Return to menu", returnToMenuPos, Color.Red);
            spriteBatch.Draw(RectTex, returnToMenuRect, Color.Transparent);
        }

        public void DrawImages(SpriteBatch spriteBatch)
        {
            for (int i = 0; i < 4; i++)
            {
                spriteBatch.Draw(textures[i], Rectangles[i], Color.White);
            }

            int k = 4;

            for (int j = 0; j < textures.Count; j++)
            {
                if (j > 3)
                {
                    spriteBatch.Draw(textures[j], Rectangles[k], Color.White);
                    k++;
                }
            }
        }
    }
}
