using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SpaceInvadersPhone
{
    public class Laser
    {
        // Texture representing the laser
        public Texture2D laserTexture;

        // Position of the laser relative to the upper left side of the screen
        public Vector2 Position;

        // State of the laser
        public bool Active;

        //Represents the viewable boundary of the game window
        Viewport viewport;

        public Rectangle laserRect;

        //Get the width of the player ship
        public int Width
        {
            get { return laserTexture.Width; }
        }

        //Get the height of the player ship
        public int Height
        {
            get { return laserTexture.Height; }
        }

        //Speed of laser
        float laserSpeed;

        /// <summary>
        /// This is the class constructor 
        /// </summary>
        /// <param name="viewport"></param>
        /// <param name="texture">The laser texture</param>
        /// <param name="position">The position of the laser relative to the players position</param>
        public void Initialize(Viewport viewport, Texture2D texture, Vector2 position)
        {
            laserTexture = texture;
            Position = position;
            this.viewport = viewport;

            Active = true;

            laserSpeed = 5f;

            float laserAspectRatio = (float)laserTexture.Width / laserTexture.Height;

            float laserWidth = (float)viewport.Width / 170;

            float laserHeight = laserWidth / laserAspectRatio;

            //Initialise the size of the player rectangle
            laserRect = new Rectangle((int)Position.X, (int)Position.Y, (int)laserWidth, (int)laserHeight);
        }

        /// <summary>
        /// Update the laser so that it travels up the screen
        /// </summary>
        public void Update()
        {
            laserRect.Y -= (int)laserSpeed;

            if (laserRect.Y + laserRect.Height <= 0)
            {
                Active = false;
            }          
        }

        /// <summary>
        /// Draw the laser to the game screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(laserTexture, laserRect, Color.White);
        }

    }
}
