using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Collections.Generic;
using System.Diagnostics;

namespace SpaceInvadersPhone
{
    public class Menu
    {
        //Main menu text position
        public Vector2 txtPos, txtPos1, txtPos2, txtPos3, txtPos4;
        public Vector2 subTextPos, subTextPos1, subTextPos2;
        public Vector2 helpPos, helpPos1;

        //Main menu text colour
        public Color colourRed, colourWhite;
        public Color option1, option2, option3, option4;

        public Rectangle rect, rect1, rect2, rect3, rect4, subRect, subRect1, subRect2;

        public List<Rectangle> rectangles, subMenuRectangles; 

        //Have added an options button to allow the player to make the game full screen

        /// <summary>
        /// Menu option highlighted i.e. 0 = "Play" option has been selected
        /// </summary>
        public int MenuPosition = 0, SubMenuPosition = 1;

        public int count = 0;

        public Boolean updated = false;

        SpriteFont currentFont, currentFont1;

        /// <summary>
        /// Initialise the main menu object along with the text positions etc..
        /// </summary>
        /// <param name="viewport">GraphicsDevice.Viewport</param>
        public void Initialize(Viewport viewport)
        {
            //Initialise colours for menu fonts
            colourRed = new Color(255, 0, 0);
            colourWhite = new Color(255, 255, 255);
            option1 = new Color(0, 0, 0);
            option2 = new Color(0, 0, 0);
            option3 = new Color(0, 0, 0);
            option4 = new Color(0, 0, 0);

            option1 = colourRed;
            option2 = colourWhite;
            option3 = colourWhite;
            option4 = colourWhite;

            txtPos.X = viewport.Width / (float)2.5;
            txtPos.Y = viewport.Height / (float)3.5;            

            txtPos1.X = viewport.Width / (float)2.5;
            txtPos1.Y = viewport.Height / (float)2.8;

            txtPos2.X = viewport.Width / (float)2.5;
            txtPos2.Y = viewport.Height / (float)2.3;

            txtPos3.X = viewport.Width / (float)2.5;
            txtPos3.Y = viewport.Height / (float)2;

            helpPos.X = 0;
            helpPos.Y = viewport.Height / (float)1.2;

            helpPos1.X = 0;
            helpPos1.Y = viewport.Height / (float)1.1;

            subTextPos.X = viewport.Width / (float)2;
            subTextPos.Y = viewport.Height / (float)4.5;

            subTextPos1.X = viewport.Width / (float)2;
            subTextPos1.Y = viewport.Height / (float)3.5;

            subTextPos2.X = viewport.Width / (float)2.5;
            subTextPos2.Y = viewport.Height / (float)6;

            rectangles = new List<Rectangle>();
            subMenuRectangles = new List<Rectangle>();

            float rectWidth = (float)viewport.Width / 3;

            float rectHeight = (float)viewport.Height / 40;

            //Initialise the size of the text rectangle
            rect = new Rectangle((int)txtPos.X, (int)txtPos.Y, (int)rectWidth, (int)rectHeight + +viewport.Height / 25);
            rectangles.Add(rect);


            //Initialise the size of the text rectangle
            rect1 = new Rectangle((int)txtPos1.X, (int)txtPos1.Y, (int)rectWidth, (int)rectHeight + viewport.Height / 25);
            rectangles.Add(rect1);

            //Initialise the size of the text rectangle
            rect2 = new Rectangle((int)txtPos2.X, (int)txtPos2.Y, (int)rectWidth, (int)rectHeight + viewport.Height / 25);
            rectangles.Add(rect2);


            //Initialise the size of the text rectangle
            rect3 = new Rectangle((int)txtPos3.X, (int)txtPos3.Y, (int)rectWidth, (int)rectHeight + viewport.Height / 25);
            rectangles.Add(rect3);

            //Initialise the size of the text rectangle
            subRect = new Rectangle((int)subTextPos.X, (int)subTextPos.Y, (int)rectWidth, (int)rectHeight + viewport.Height / 25);
            subMenuRectangles.Add(subRect);

            //Initialise the size of the text rectangle
            subRect1 = new Rectangle((int)subTextPos1.X, (int)subTextPos1.Y, (int)rectWidth, (int)rectHeight + +viewport.Height / 25);
            subMenuRectangles.Add(subRect1);

            subRect2 = new Rectangle((int)subTextPos2.X, (int)subTextPos2.Y, (int)rectWidth, (int)rectHeight + +viewport.Height / 25);
            subMenuRectangles.Add(subRect2);

            subMenuRectangles.Add(rect1);
            subMenuRectangles.Add(rect2);
            subMenuRectangles.Add(rect3);
        }

        /// <summary>
        /// Expand the menu option when clicked
        /// </summary>
        /// <param name="viewport"></param>
        public void ExpandMenuOption(Viewport viewport)
        {
            //move the option up the screen to make room for the other options
            txtPos.X = viewport.Width / (float)2.5;
            txtPos.Y = viewport.Height / (float)6;
        }

        
        public void DrawBackground(SpriteBatch spriteBatch, Texture2D background, Viewport viewport)
        {
            float AspectRatio = (float)background.Width / background.Height;

            float Width = (float)viewport.Width;

            float Height = Width / AspectRatio;


            Rectangle backRect = new Rectangle(0, 0, (int)Width, (int)Height);
            spriteBatch.Draw(background, backRect, Color.White);
        }

        public void DrawSubMenu(SpriteBatch spriteBatch, SpriteFont font, SpriteFont font1, int position, Texture2D rectTex)
        {            
            for (int i = 0; i < subMenuRectangles.Count; i++)
            {
                spriteBatch.Draw(rectTex, subMenuRectangles[i], Color.Transparent);
            }

            if (position == 1)
            {
                spriteBatch.DrawString(font1, "Classic", subTextPos, Color.Red);
                spriteBatch.DrawString(font, "Modern", subTextPos1, Color.White);
            }

            else if (position == 2)
            {
                spriteBatch.DrawString(font, "Classic", subTextPos, Color.White);
                spriteBatch.DrawString(font1, "Modern", subTextPos1, Color.Red);
            }
        }

        public void Draw(SpriteBatch spriteBatch, SpriteFont font, SpriteFont font1, int position, Texture2D rectTex)
        {
            for(int i = 0; i < rectangles.Count; i++)
            {
                spriteBatch.Draw(rectTex, rectangles[i], Color.Transparent);
            }

            spriteBatch.DrawString(font, "Play", txtPos, Color.White);
            spriteBatch.DrawString(font, "Instructions", txtPos1, Color.White);
            spriteBatch.DrawString(font, "Controls", txtPos2, Color.White);
            spriteBatch.DrawString(font, "Exit", txtPos3, Color.White);          
        }

        public void DrawNavigation(SpriteBatch spriteBatch, SpriteFont font)
        {
            spriteBatch.DrawString(font, "Enter/A to make selection", helpPos, Color.White);
            spriteBatch.DrawString(font, "Escape/B to go back", helpPos1, Color.White);            
        }
    }
}
