using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SpaceInvadersPhone
{
    class PausedScreen
    {
        public Vector2 txtPos, txtPos1, txtPos2;

        //Text colours
        public Color colourRed, colourWhite;

        //The paused screen texture
        public Texture2D pausedScreen, RectangleTex;

        //Rectangles for user clicking
        public Rectangle ResumeRect, ExitRect;

        // Get the width of the player ship
        public int PausedWidth
        {
            get { return pausedScreen.Width; }
        }

        // Get the height of the player ship
        public int PausedHeight
        {
            get { return pausedScreen.Height; }
        }

        //Rectangle of the game screen
        Rectangle pausedRect;

        /// <summary>
        /// Initialise the paused screen object along with the text positions etc..
        /// </summary>
        /// <param name="viewport">GraphicsDevice.Viewport</param>
        public void Initialize(Viewport viewport, Texture2D rectTex)
        {
            //Initialise colours for menu fonts
            colourRed = new Color(255, 0, 0);
            colourWhite = new Color(255, 255, 255);

            //Initialise the text positions 
            txtPos.X = viewport.Width / (float)2.5;
            txtPos.Y = viewport.Height / (float)12;

            txtPos1.X = viewport.Width / (float)2.5;
            txtPos1.Y = viewport.Height / (float)3.25;

            txtPos2.X = viewport.Width / (float)2.5;
            txtPos2.Y = viewport.Height / (float)2;

            float rectWidth = (float)viewport.Width / 3;

            float rectHeight = (float)viewport.Height / 40;

            ResumeRect = new Rectangle((int)txtPos2.X, (int)txtPos1.Y, (int)rectWidth, (int)rectHeight + viewport.Height / 25);
            ExitRect = new Rectangle((int)txtPos2.X, (int)txtPos1.Y, (int)rectWidth, (int)rectHeight + viewport.Height / 25);
            RectangleTex = rectTex;
        }

        /// <summary>
        /// Draw the paused screen over the game screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        /// <param name="font"></param>        
        public void DrawPausedScreen(SpriteBatch spriteBatch, SpriteFont font)
        {
            spriteBatch.DrawString(font, "Paused", txtPos, colourRed);
            spriteBatch.DrawString(font, "Exit", txtPos1, colourWhite);
            spriteBatch.DrawString(font, "Resume", txtPos2, colourWhite);

            spriteBatch.Draw(RectangleTex, ResumeRect, Color.Transparent);
            spriteBatch.Draw(RectangleTex, ExitRect, Color.Transparent);
        }

    }
}
