using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SpaceInvadersPhone
{
    public class Player
    {
        /// <summary>
        ///  Animation representing the player
        /// </summary>        

        public Texture2D PlayerTexture, PlayerTexture1, PlayerTexture2, PlayerTextureShielded, PlayerTexture1Shielded, PlayerTexture2Shielded;


        /// <summary>
        /// Position of the Player relative to the upper left side of the screen
        /// </summary>
        /// 
        public Vector2 Position;

        /// <summary>
        /// Rectangle that scales the size of the player texture to equal 10% of the screen width and height
        /// </summary>
        public Rectangle playerRect;


        //The speed at which the player moves
        public float playerMoveSpeed;

        // State of the player
        public bool Active;

        // Amount of hit points that player has
        public int Lives = 3;

        //Player score
        public int Score = 0;

        //Round number
        public int Round = 1;

        // Get the width of the player ship
        public int Width
        {
            get { return PlayerTexture.Width; }
        }

        // Get the height of the player ship
        public int Height
        {
            get { return PlayerTexture.Height; }
        }

        bool gameOver = false;

        public bool dualLasers = false, movement = false, shields = false;

        /// <summary>
        /// The constructor sets all possible textures for the player along with its position
        /// </summary>
        /// <param name="texture">The non moving player texture</param>
        /// <param name="texture1">The moving left player texture</param>
        /// <param name="texture2">The moving right player texture</param>
        /// <param name="shield">The shield texture</param>
        /// <param name="viewport">Viewport</param>

        public void Initialize(Texture2D texture, Texture2D texture1, Texture2D texture2, Texture2D texture3, Texture2D texture4, Texture2D texture5, Viewport viewport)
        {
            PlayerTexture = texture;
            PlayerTexture1 = texture1;
            PlayerTexture2 = texture2;
            PlayerTextureShielded = texture3;
            PlayerTexture1Shielded = texture4;
            PlayerTexture2Shielded = texture5;

            playerMoveSpeed = 6.0f;

            // Set the starting position of the player around the middle of thescreen and to the back                    
            Position.X = ((viewport.Width / 2) - playerRect.Width);
            Position.Y = (viewport.Height / (float)1.05);

            // Set the player to be active
            Active = true;
           
            float playerAspectRatio = (float)Width / Height;

            float playerWidth = (float)viewport.Width / 40;

            float playerHeight = playerWidth / playerAspectRatio;

            //Initialise the size of the player rectangle
            playerRect = new Rectangle((int)Position.X, (int)Position.Y, (int)playerWidth, (int)playerHeight);
        }

        /// <summary>
        /// Make the player object lose a life
        /// </summary>
        /// <returns>Number of lives remaining</returns>
        public Boolean LoseLife()
        {
            Lives--;

            if (Lives == 0)
            {
                gameOver = true;
            }

            return gameOver;
        }

        /// <summary>
        /// Draws the player when it is stationary
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(PlayerTexture, playerRect, Color.White);
        }

        /// <summary>
        /// Draws the player texture when it is moving left
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void DrawLeft(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(PlayerTexture1, playerRect, Color.White);
        }

        /// <summary>
        /// Draws the player texture when it is moving right 
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void DrawRight(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(PlayerTexture2, playerRect, Color.White);
        }

        /// <summary>
        /// Draws the player when it is stationary
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void DrawShield(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(PlayerTextureShielded, playerRect, Color.White);
        }

        /// <summary>
        /// Draws the player texture when it is moving left
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void DrawLeftShield(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(PlayerTexture1Shielded, playerRect, Color.White);
        }

        /// <summary>
        /// Draws the player texture when it is moving right 
        /// </summary>
        /// <param name="spriteBatch"></param>
        public void DrawRightShield(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(PlayerTexture2Shielded, playerRect, Color.White);
        }
    }
}

